#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
#  \file  CorbaPort.py
#  \brief CorbaPort class
#  \date  $Date: 2007/09/26 $
#  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
# 
#  Copyright (C) 2006-2008
#      Noriaki Ando
#      Task-intelligence Research Group,
#      Intelligent Systems Research Institute,
#      National Institute of
#          Advanced Industrial Science and Technology (AIST), Japan
#      All rights reserved.

from omniORB import CORBA
from omniORB import any
import traceback
import sys

import OpenRTM_aist
import RTC, RTC__POA



##
# @if jp
# @class CorbaPort
# @brief RT ݡͥ CORBA provider/consumer  Port
#
# CorbaPort  RT ݡͥȤˤơ桼 CORBA ֥
# ӥӥ󥷥塼ޤ󶡤 Port Ǥ롣
# <p>
# RT ݡͥȤϡPort 𤷤ƥ桼 CORBA ӥ
# 뤳ȤǤ RT Service (Provider) ȸƤ֡
# ޤ¾ RT ݡͥȤΥӥѤ뤿 CORBA ֥
# Υץ졼ۥ󶡤뤳ȤǤ RT Service Consumer ȸƤ֡
# <p>
# CorbaPort Ǥդο Provider  Consumer 뤳ȤǤ
# Port ƱΤ³ݤб Provider  Consumer Ŭڤ˴Ϣդ
# ȤǤ롣
# <p>
# CorbaPort ̾ʲΤ褦Ѥ롣
#
# <pre>
# # CORBAݡȤν
# self._myServicePort = OpenRTM_aist.CorbaPort("MyService")
#
# // Provider¦
# #  Port 󶡤 Serivce Provider ν
# self._mpros = MyServiceSVC_impl()
# # Service Provider  Port Ͽ
# self._myServicePort.registerProvider("myservice0", "MyService", self._mpros)
#
# // Consumer¦
# #  Port 󶡤 Serivce Consumer ν
# self._mycons = OpenRTM_aist.CorbaConsumer(interfaceType=_GlobalIDL.MyService)
# # Service Consumer  Port Ͽ
# self._myServicePort.registerConsumer("myservice0", "MyService", self._mycons)
#
#
# # CORBAݡȤؤϿ
# self.registerPort(self._myServicePort)
#
# // connect Ԥ줿
# self.my_cons._ptr().your_service_function(); // YourService δؿ򥳡
#
# // connect 줿 ̤ΥݡͥȤˤ
# self.m_cons1._ptr().my_service_function(); // MyService δؿ򥳡
# </pre>
#
# Τ褦ˡ󶡤 Service Provider  registerProvider() Ͽ
# 뤳Ȥˤꡢ¾ΥݡͥȤѲǽˤ¾
# Ѥ Service Consumer  registerConsumer() Ͽ뤳Ȥˤ
# ¾ΥݡͥȤ Service 򥳥ݡͥѲǽˤ뤳Ȥ
# Ǥ롣
#
# @since 0.4.0
#
# @else
# @class CorbaPort
# @brief RT Conponent CORBA service/consumer Port
#
# CorbaPort is an implementation of the Port of RT-Component's that provides
# user-defined CORBA Object Service and Consumer.
# <p>
# RT-Component can provide user-defined CORBA serivces, which is called
# RT-Serivce (Provider), through the Ports.
# RT-Component can also provide place-holder, which is called RT-Serivce
# Consumer, to use other RT-Component's service.
# <p>
# The CorbaPort can manage any number of Providers and Consumers, can
# associate Consumers with correspondent Providers when establishing
# connection among Ports.
# <p>
# Usually, CorbaPort is used like the following.
#
# <pre>
# RTC::CorbaPort m_port0; // declaration of Port
#
# MyService_impl m_mysvc0; // Serivce Provider that is provided by the Port
# RTC::CorbaConsumer<YourService> m_cons0; // Consumer of the Port
#
# // register Service Provider to the Port
# m_port0.registerProvider("MyService0", "Generic", m_mysvc0);
# // register Service Consumer to the Port
# m_port0.registerConsumer("YourService0", "Generic", m_cons0 );
#
# // after connect established
#
# m_cons0->your_service_function(); // call a YourService's function
#
# // in another component that is connected with the Port
# m_cons1->my_service_function(); // call a MyService's function
# </pre>
#
# Registering Service Provider by registerProvider(), it can be used from
# other RT-Components.
# Registering Service Consumer by registerConsumer(), other RT-Component's
# services can be used through the consumer object.
#
# @since 0.4.0
#
# @endif
class CorbaPort(OpenRTM_aist.PortBase):
  """
  """

  ##
  # @if jp
  # @brief ProviderξǼ빽¤
  # @else
  # @brief The structure to be stored Provider's information.
  # @endif
  #
  class ProviderInfo:
    def __init__(self, _servant, _objectid):
      self.servant = _servant
      self.oid     = _objectid

  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # @param self
  # @param name Port ̾
  #
  # @else
  #
  # @brief Constructor
  #
  # @param name The name of Port 
  #
  # @endif
  def __init__(self, name):
    OpenRTM_aist.PortBase.__init__(self, name)
    self.addProperty("port.port_type", "CorbaPort")
    self._providers = []
    self._servants  = {}
    self._consumers = []


  ##
  # @if jp
  #
  # @brief Provider Ͽ
  #
  #  Port ˤ󶡤ХȤ򤳤 Port ФϿ롣
  # ХȤϡͿ instance_name, type_name 
  # ХȼȤΥ󥹥̾ӥ̾ȤơХȤ
  # Ϣդ롣
  #
  # @param self
  # @param instance_name ХȤΥ󥹥̾
  # @param type_name ХȤΥ̾
  # @param provider CORBA Х
  #
  # @return Ʊ̾ instance_name ϿƤ false ֤
  #
  # @else
  #
  # @brief Register provider
  #
  # This operation registers a servant, which is provided in this Port,
  # to the Port. The servant is associated with "instance_name" and
  # "type_name" as the instance name of the servant and as the type name
  # of the servant.
  #
  # @param self
  # @param instance_name Servant instance name
  # @param type_name Servant type name
  # @param provider CORBA Servant
  #
  # @return if same instance_name is registered, this will return False
  #
  # @endif
  def registerProvider(self, instance_name, type_name, provider):
    self._rtcout.RTC_TRACE("registerProvider(instance=%s, type_name=%s)",
                           (instance_name, type_name))

    if not self.appendInterface(instance_name, type_name, RTC.PROVIDED):
      return False

    oid = OpenRTM_aist.Manager.instance().getPOA().servant_to_id(provider)

    try:
      OpenRTM_aist.Manager.instance().getPOA().activate_object_with_id(oid,provider)
    except:
      pass
    obj = OpenRTM_aist.Manager.instance().getPOA().id_to_reference(oid)

    key = "port"
    key = key + "." + str(type_name) + "." + str(instance_name)

    orb = OpenRTM_aist.Manager.instance().getORB()
    ior = orb.object_to_string(obj)
    OpenRTM_aist.CORBA_SeqUtil.push_back(self._providers,
                                         OpenRTM_aist.NVUtil.newNV(key, ior))
    self._servants[instance_name] = self.ProviderInfo(provider,oid)

    return True


  ##
  # @if jp
  #
  # @brief Consumer Ͽ
  #
  #  Port ׵᤹륵ӥΥץ졼ۥǤ륳󥷥塼
  # (Consumer) Ͽ롣
  # Consumer Ϣդ륵ӥΥ󥹥̾ӥ̾Ȥơ
  #  instance_name, type_name  Consumer ȤͿ뤳Ȥˤꡢ
  # Ǥ餬Ϣդ롣
  # Port ֤³ (connect)  ˤϡƱ instance_name, type_name 
  # ӥ¾ Port  (Provide) Ƥ硢Υӥ
  # ֥ȻȤưŪ Consumer ˥åȤ롣
  #
  # @param self
  # @param instance_name Consumer ׵᤹륵ӥΥ󥹥̾
  # @param type_name Consumer ׵᤹륵ӥΥ̾
  # @param consumer CORBA ӥ󥷥塼
  #
  # @return Ʊ̾ instance_name ϿƤ false ֤
  #
  # @else
  #
  # @brief Register consumer
  #
  # This operation registers a consumer, which requiers a service,
  # to the other Port. The consumer is associated with "instance_name" and
  # "type_name" as the instance name of the service and as the type name
  # of the service that is required.
  #
  # @param self
  # @param instance_name An instance name of the service required
  # @param type_name An type name of the service required
  # @param consumer CORBA service consumer
  #
  # @return False would be returned if the same instance_name is registered
  #
  # @endif
  def registerConsumer(self, instance_name, type_name, consumer):
    if not self.appendInterface(instance_name, type_name, RTC.REQUIRED):
      return False

    cons = self.Consumer(instance_name, type_name, consumer)
    self._consumers.append(cons)

    return True


  ##
  # @if jp
  # @brief Port ƤΥ󥿡ե activates 
  # @else
  # @brief Activate all Port interfaces
  # @endif
  #
  # void CorbaPort::activateInterfaces()
  def activateInterfaces(self):
    for key in self._servants.keys():
      try:
        OpenRTM_aist.Manager.instance().getPOA().activate_object_with_id(self._servants[key].oid, self._servants[key].servant)
      except:
        pass

  
  ##
  # @if jp
  # @brief Ƥ Port Υ󥿡ե deactivates 
  # @else
  # @brief Deactivate all Port interfaces
  # @endif
  #
  # void CorbaPort::deactivateInterfaces()
  def deactivateInterfaces(self):
    for key in self._servants.keys():
      OpenRTM_aist.Manager.instance().getPOA().deactivate_object(self._servants[key].oid)


  ##
  # @if jp
  #
  # @brief Interface 
  #
  #  Portͭ Provider ˴ؤ ConnectorProfile::properties
  # 롣
  # ϡNVList name  value ȤưʲΤΤǼ롣
  #
  # - port.<type_name>.<instance_name>: <CORBA::Object_ptr>
  #
  # ǡ
  # - <type_name>: PortInterfaceProfile::type_name
  # - <instance_name>: PortInterfaceProfile::instance_name<br>
  # Ǥ롣<br>
  # ConnectorProfile::properties Ǥϡ .(ɥå)ɽǡ
  # NameValue ΥȤƤ롣äơ
  #
  # <pre>
  #  PortInterfaceProfile
  #  {
  #    instance_name = "PA10_0";
  #    type_name     = "Manipulator";
  #    polarity      = PROVIDED;
  #  }
  #</pre>
  #
  # ʤС
  #
  # <pre>
  # NameValue = { "port.Manipulator.PA10_0": <Object reference> }
  # </pre>
  #
  # Ȥäͤ ConnectorProfile::properties ˳Ǽ졢¾ΥݡȤФ
  # ã롣¾ Port ǤΥ󥿡եѤ Consumer 
  # ¸ߤСConnectorProfile 餳Υ饪֥ȥե󥹤
  # 餫ηǻѤ롣
  #
  # @param self
  # @param connector_profile ͥץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Publish interface information
  #
  # @endif
  def publishInterfaces(self, connector_profile):
    self._rtcout.RTC_TRACE("publishInterfaces()")
    OpenRTM_aist.CORBA_SeqUtil.push_back_list(connector_profile.properties,
                                              self._providers)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief Interface ³
  #
  #  Portͭ Consumer Ŭ礹 Provider ˴ؤ
  # ConnectorProfile::properties Ф Consumer ˥֥Ȼ
  # 򥻥åȤ롣
  #
  # Consumer 
  # <pre>
  #  PortInterfaceProfile
  #  {
  #    instance_name = "PA10_0";
  #    type_name     = "Manipulator";
  #    polarity      = REQUIRED;
  #  }
  # </pre>
  # ȤϿƤС¾ Port 
  # <pre>
  #  PortInterfaceProfile
  #  {
  #    instance_name = "PA10_0";
  #    type_name     = "Manipulator";
  #    polarity      = PROVIDED;
  #  }
  # </pre> 
  # ȤϿƤ Serivce Provider Υ֥ȻȤõ
  # Consumer ˥åȤ롣
  # ºݤˤϡConnectorProfile::properties 
  # <pre>
  # NameValue = { "port.Manipulator.PA10_0": <Object reference> }
  # </pre>
  # ȤϿƤ NameValue õΥ֥ȻȤ
  # Consumer ˥åȤ롣
  #
  # @param self
  # @param connector_profile ͥץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Subscribe interfaces
  #
  # @endif
  def subscribeInterfaces(self, connector_profile):
    self._rtcout.RTC_TRACE("subscribeInterfaces()")
    nv = connector_profile.properties

    orb = OpenRTM_aist.Manager.instance().getORB()
    
    for cons in self._consumers:
      index = OpenRTM_aist.NVUtil.find_index(nv, cons.name)
      if index < 0:
        continue

      ior = any.from_any(nv[index].value, keep_structs=True)
      if not ior:
        self._rtcout.RTC_WARN("Cannot extract IOR string")
        continue

      obj = orb.string_to_object(ior)

      if CORBA.is_nil(obj):
        self._rtcout.RTC_ERROR("Extracted object is null reference")
        continue

      result = cons.consumer.setObject(obj)

      if not result:
        self._rtcout.RTC_ERROR("Cannot narrow reference")
        continue

    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief Interface ؤ³
  #
  # Ϳ줿 ConnectorProfile ˴Ϣ Consumer ˥åȤ줿
  # ٤Ƥ Object ³롣
  #
  # @param self
  # @param connector_profile ͥץե
  #
  # @else
  #
  # @brief Unsubscribe interfaces
  #
  # @endif
  def unsubscribeInterfaces(self, connector_profile):
    self._rtcout.RTC_TRACE("unsubscribeInterfaces()")
    nv = connector_profile.properties

    OpenRTM_aist.CORBA_SeqUtil.for_each(nv, self.unsubscribe(self._consumers))



  ##
  # @if jp
  # @brief Consumer ξǼ빽¤
  # @else
  # @brief Consumer inforamtion struct
  # @endif
  class Consumer:
    def __init__(self, _instance_name, _type_name, _cons, _consumer=None):
      if _consumer:
        self.name = _consumer.name
        self.consumer = _consumer.consumer
        return
      
      self.name = "port."+str(_type_name)+"."+str(_instance_name)
      self.consumer = _cons



  ##
  # @if jp
  # @brief ConnectorProfile  Consuemr Ӥ򤷥֥ȻȤ
  #        åȤ뤿 Functor
  # @else
  # @brief Subscription mutching functor for Consumer
  # @endif
  class subscribe:
    def __init__(self, cons):
      self._cons = cons
      self._len  = len(cons)

    def __call__(self, nv):
      for i in range(self._len):
        name_ = nv.name
        if self._cons[i].name == name_:
          try:
            obj = any.from_any(nv.value, keep_structs=True)
            self._cons[i].consumer.setObject(obj)
          except:
            traceback.print_exception(*sys.exc_info())



  ##
  # @if jp
  # @brief Consumer Υ֥Ȥ뤿 Functor
  # @else
  # @brief Unsubscription functor for Consumer
  # @endif
  class unsubscribe:
    def __init__(self, cons):
      self._cons = cons
      self._len  = len(cons)

    def __call__(self, nv):
      for i in range(self._len):
        name_ = nv.name
        if self._cons[i].name == name_:
          self._cons[i].consumer.releaseObject()


