#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file ManagerConfig.py
# @brief RTC manager configuration
# @date $Date: $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2003-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#    All rights reserved.


import sys
import os
import re
import getopt
import platform

import OpenRTM_aist


##
# @if jp
#
# @class ManagerConfig
# @brief Manager configuration 饹
#
# Manager Υե졼Ԥޥɥ饤ꡢ
# (뤤ϰʤ)󥹥󥹲롣
# ޥɥ饤ǻꤵ줿ե롢Ķѿʤɤ Manager 
# ץѥƥꤹ롣
#
# ͥ٤ϰʲΤȤǤ롣
# <OL>
# <LI>ޥɥ饤󥪥ץ "-f"
# <LI>Ķѿ "RTC_MANAGER_CONFIG"
# <LI>ǥեե "./rtc.conf"
# <LI>ǥեե "/etc/rtc.conf"
# <LI>ǥեե "/etc/rtc/rtc.conf"
# <LI>ǥեե "/usr/local/etc/rtc.conf"
# <LI>ǥեե "/usr/local/etc/rtc/rtc.conf"
# <LI>ߥե졼
#</OL>
# ޥɥ饤󥪥ץ "-d" ꤵ줿ϡ
# (Ȥ -f եꤷƤ)ߥե졼
# Ѥ롣
#
# @since 0.4.0
#
# @else
#
# @brief
#
# @endif
class ManagerConfig :
  """
  """



  ##
  # @if jp
  # @brief Manager ե졼Υǥեȡե롦ѥ
  # @else
  # @endif
  config_file_path = ["./rtc.conf",
            "/etc/rtc.conf",
            "/etc/rtc/rtc.conf",
            "/usr/local/etc/rtc.conf",
            "/usr/local/etc/rtc/rtc.conf",
            None]


  ##
  # @if jp
  # @brief ǥեȡե졼Υե롦ѥ̤
  #        Ķѿ
  # @else
  # @endif
  config_file_env = "RTC_MANAGER_CONFIG"


  ##
  # @if jp
  #
  # @brief 󥹥ȥ饯
  #
  # Ϳ줿ˤꥳե졼νԤ
  #
  # @param self
  # @param argv ޥɥ饤(ǥե:None)
  #
  # @else
  #
  # @brief ManagerConfig constructor
  #
  # The constructor that performs initialization at the same time with
  # given arguments.
  #
  # @param argv The command line arguments
  #
  # @endif
  def __init__(self, argv=None):

    self._configFile = ""
    if argv:
      self.init(argv)


  ##
  # @if jp
  #
  # @brief 
  #
  # ޥɥ饤˱ƽ¹Ԥ롣ޥɥ饤󥪥ץ
  # ʲΤΤѲǽǤ롣
  #
  # -f file   : ե졼եꤹ롣<br>
  # -l module : ɤ⥸塼ꤹ롣(̤)<br>
  # -o options: ¾ץꤹ롣(̤)<br>
  # -d        : ǥեȤΥե졼Ȥ(̤)<br>
  #
  # @param self
  # @param argv ޥɥ饤
  #
  # @else
  #
  # @brief Initialization
  #
  # Initialize with command line options. The following command options
  # are available.
  #
  # -f file   : Specify a configuration file. <br>
  # -l module : Specify modules to be loaded at the beginning. <br>
  # -o options: Other options. <br>
  # -d        : Use default static configuration. <br>
  #
  # @endif
  def init(self, argv):
    self.parseArgs(argv)


  ##
  # @if jp
  # @brief Configuration  Property ꤹ
  # 
  # Manager Configuration ꤵ줿 Property ꤹ롣
  #
  # @param self
  # @param prop Configuration о Property
  # 
  # @else
  # @brief Apply configuration results to Property
  # @endif
  def configure(self, prop):
    prop.setDefaults(OpenRTM_aist.default_config)
    if self.findConfigFile():
      try:
        fd = file(self._configFile,"r")
        prop.load(fd)
        fd.close()
      except:
        print "Error: file open."
    return self.setSystemInformation(prop)

  #######
  # \if jp
  #
  # \brief ե졼(̤)
  #
  # ե졼롣init()ƤӽФ˸Ƥ֤ȡ
  # Ū줿ǥեȤΥե졼֤
  # init() ƤӽФ˸Ƥ֤ȡޥɥ饤Ķѿ
  # Ť줿ե졼֤
  #
  # \else
  #
  # \brief Get configuration value.
  #
  # This operation returns default configuration statically defined,
  # when before calling init() function. When after calling init() function,
  # this operation returns initialized configuration value according to
  # command option, environment value and so on.
  #
  # \endif
  #def getConfig(self):
  #pass


  ##
  # @if jp
  #
  # @brief ޥɰѡ
  #
  # -f file   : ե졼եꤹ롣<br>
  # -l module : ɤ⥸塼ꤹ롣(̤)<br>
  # -o options: ¾ץꤹ롣(̤)<br>
  # -d        : ǥեȤΥե졼Ȥ(̤)<br>
  #
  # @param self
  # @param argv ޥɥ饤
  #
  # @else
  #
  # @brief Parse command arguments
  #
  # -f file   : Specify a configuration file. <br>
  # -l module : Specify modules to be loaded at the beginning. <br>
  # -o options: Other options. <br>
  # -d        : Use default static configuration. <br>
  #
  # @endif
  def parseArgs(self, argv):
    try:
      opts, args = getopt.getopt(argv[1:], "f:l:o:d:")
    except getopt.GetoptError:
      print "Error: getopt error!"
      return

    for opt, arg in opts:
      if opt == "-f":
        self._configFile = arg

      if opt == "-l":
        pass

      if opt == "-o":
        pass

      if opt == "-d":
        pass

    return


  ##
  # @if jp
  #
  # @brief Configuration file θ
  #
  # Configuration file 򸡺ꤹ롣
  #  Configuration file Ѥߤξϡե¸߳ǧԤ
  #
  # Configuration file ͥ<br>
  # ޥɥץĶѿǥեȥեǥե
  #
  # ǥեȶץ(-d): ǥեȥե뤬äƤ̵뤷
  #                               ǥեȤ
  #
  # @param self
  #
  # @return Configuration file 
  #
  # @else
  #
  # @brief Find configuration file
  #
  # @endif
  def findConfigFile(self):
    if self._configFile != "":
      if self.fileExist(self._configFile):
        return True

    env = os.getenv(self.config_file_env)
    if env:
      if self.fileExist(env):
        self._configFile = env
        return True

    i = 0
    while (self.config_file_path[i]):
      if self.fileExist(self.config_file_path[i]):
        self._configFile = self.config_file_path[i]
        return True
      i += 1

    return False


  ##
  # @if jp
  #
  # @brief ƥꤹ
  #
  # ƥץѥƥ˥åȤ롣ꤵ륭ϰʲ̤ꡣ
  #  - manager.os.name    : OS̾
  #  - manager.os.release : OS꡼̾
  #  - maanger.os.version : OSС̾
  #  - manager.os.arch    : OSƥ
  #  - manager.os.hostname: ۥ̾
  #  - manager.pid        : ץID
  # 
  # @param self
  # @param prop ƥꤷץѥƥ
  #
  # @else
  # 
  # @brief Set system information
  # 
  # Get the following system info and set them to Manager's properties.
  #  - manager.os.name    : OS name
  #  - manager.os.release : OS release name
  #  - manager.os.version : OS version
  #  - manager.os.arch    : OS architecture
  #  - manager.os.hostname: Hostname
  #  - manager.pid        : process ID
  #
  # @endif
  def setSystemInformation(self, prop):
    sysinfo = platform.uname()

    prop.setProperty("manager.os.name",     sysinfo[0])
    prop.setProperty("manager.os.hostname", sysinfo[1])
    prop.setProperty("manager.os.release",  sysinfo[2])
    prop.setProperty("manager.os.version",  sysinfo[3])
    prop.setProperty("manager.os.arch",     sysinfo[4])
    prop.setProperty("manager.pid",         os.getpid())
    
    return prop


  ##
  # @if jp
  # @brief ե¸߳ǧ
  #
  # ꤵ줿ե뤬¸ߤ뤫ǧ롣
  #
  # @param self
  # @param filename ǧоݥե̾
  #
  # @return оݥեǧ(¸ߤtrue)
  #
  # @else
  # @brief Check file existance
  # @endif
  def fileExist(self, filename):
    try:
      fp = open(filename)
    except:
      print "Can't open file:", filename
      return False
    else:
      fp.close()
      return True

    return False


