#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file NamingManager.py
# @brief naming Service helper class
# @date $Date: 2007/08/27$
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2006-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.


import threading
import traceback
import sys

import OpenRTM_aist


##
# @if jp
#
# @class NamingBase
# @brief NamingService ݥ饹
#
# NamingServer ݥ󥿡ե饹
# ݴ饹ϡʲδؿμ󶡤ʤФʤʤ
# - bindObject() : ꤷ֥ȤNamingServiceؤΥХ
# - unbindObject() : ꤷ֥ȤNamingServiceΥХ
#
# @since 0.4.0
#
# @else
#
# @endif
class NamingBase:



  ##
  # @if jp
  #
  # @brief NamingServiceإХɤؿ(֥饹)
  #
  # ꤷ֥ȤNamingServiceإХɤ<BR>
  # ֥饹Ǥμ
  #
  # @param self
  # @param name Хɻ̾
  # @param rtobj Хоݥ֥
  #
  # @else
  #
  # @endif
  def bindObject(self, name, rtobj):
    pass


  ##
  # @if jp
  #
  # @brief NamingService饢Хɤؿ(֥饹)
  #
  # ꤷ֥ȤNamingService饢Хɤ<BR>
  # ֥饹Ǥμ
  #
  # @param self
  # @param name Хоݥ֥
  #
  # @else
  #
  # @endif
  def unbindObject(self, name):
    pass



##
# @if jp
#
# @class NamingOnCorba
# @brief CORBA  NamingServer 饹
#
# CORBA  NamingServer ѥ饹
# CORBA ݡͥȤNamingServiceؤϿʤɤ롣
#
# @since 0.4.0
#
# @else
#
# @biref ModuleManager class
#
# @endif
class NamingOnCorba(NamingBase):



  ##
  # @if jp
  #
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param orb ORB
  # @param names NamingServer ̾
  #
  # @else
  #
  # @endif
  def __init__(self, orb, names):
    self._cosnaming = OpenRTM_aist.CorbaNaming(orb,names)


  ##
  # @if jp
  #
  # @brief ꤷ CORBA ֥ȤNamingServiceإХ
  # 
  # ꤷ CORBA ֥Ȥꤷ̾Τ CORBA NamingService 
  # Хɤ롣
  # 
  # @param self
  # @param name Хɻ̾
  # @param rtobj Хоݥ֥
  #
  # @else
  #
  # @endif
  def bindObject(self, name, rtobj):
    try:
      self._cosnaming.rebindByString(name, rtobj.getObjRef(), True)
    except:
      pass


  ##
  # @if jp
  #
  # @brief ꤷ CORBA ֥ȤNamingService饢Х
  # 
  # ꤷ CORBA ֥Ȥ CORBA NamingService 饢Хɤ롣
  # 
  # @param self
  # @param name Хоݥ֥
  #
  # @else
  #
  # @endif
  def unbindObject(self, name):
    try:
      self._cosnaming.unbind(name)
    except:
      traceback.print_exception(*sys.exc_info())



##
# @if jp
#
# @class NamingManager
# @brief NamingServer 饹
#
# NamingServer ѥ饹
# ݡͥȤNamingServiceؤϿʤɤ롣
#
# @since 0.4.0
#
# @else
#
# @biref ModuleManager class
#
# @endif
class NamingManager:
  """
  """



  ##
  # @if jp
  #
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param manager ޥ͡㥪֥
  #
  # @else
  #
  # @endif
  def __init__(self, manager):
    self._manager = manager
    self._rtcout = manager.getLogbuf('manager.namingmanager')
    #self._rtcout.setLogLevel(manager.getConfig().getProperty("logger.log_level"))
    #self._rtcout.setLogLock(OpenRTM_aist.toBool(manager.getConfig().getProperty("logger.stream_lock"), "enable", "disable", False))
    self._names = []
    self._namesMutex = threading.RLock()
    self._compNames = []
    self._mgrNames  = []
    self._compNamesMutex = threading.RLock()
    self._mgrNamesMutex = threading.RLock()


  ##
  # @if jp
  #
  # @brief NameServer Ͽ
  #
  # ꤷ NameServer Ͽ롣
  # ߻ǽʷ CORBA Τߡ
  #
  # @param self
  # @param method NamingService η
  # @param name_server Ͽ NameServer ̾
  #
  # @else
  #
  # @endif
  def registerNameServer(self, method, name_server):
    self._rtcout.RTC_TRACE("NamingManager::registerNameServer(%s, %s)",
                           (method, name_server))
    name = self.createNamingObj(method, name_server)
    self._names.append(self.Names(method, name_server, name))


  ##
  # @if jp
  #
  # @brief ꤷ֥ȤNamingServiceإХ
  # 
  # ꤷ֥Ȥꤷ̾Τ CORBA NamingService إХɤ롣
  # 
  # @param self
  # @param name Хɻ̾
  # @param rtobj Хоݥ֥
  #
  # @else
  #
  # @endif
  def bindObject(self, name, rtobj):
    self._rtcout.RTC_TRACE("NamingManager::bindObject(%s)", name)
    guard = OpenRTM_aist.ScopedLock(self._namesMutex)
    for i in range(len(self._names)):
      if self._names[i].ns:
        self._names[i].ns.bindObject(name, rtobj)
    self.registerCompName(name, rtobj)


  def bindManagerObject(self, name, mgr):
    self._rtcout.RTC_TRACE("NamingManager::bindManagerObject(%s)", name)
    guard = OpenRTM_aist.ScopedLock(self._namesMutex)
    for i in range(len(self._names)):
      if self._names[i].ns:
        self._names[i].ns.bindObject(name, mgr)
    self.registerMgrName(name, mgr)


  ##
  # @if jp
  #
  # @brief NamingServer ξι
  # 
  # ꤵƤ NameServer ϿƤ륪֥Ȥξ
  # 롣
  # 
  # @param self
  # 
  # @else
  #
  # @endif
  def update(self):
    self._rtcout.RTC_TRACE("NamingManager::update()")
    guard = OpenRTM_aist.ScopedLock(self._namesMutex)
    for i in range(len(self._names)):
      if self._names[i].ns is None:
        nsobj = self.createNamingObj(self._names[i].method,
                                     self._names[i].nsname)
        if nsobj:
          self._rtcout.RTC_INFO("New name server found: %s/%s",
                                (self._names[i].method,
                                 self._names[i].nsname))
          self._names[i].ns = nsobj
          self.bindCompsTo(nsobj)
      else:
        self.bindCompsTo(self._names[i].ns)


  ##
  # @if jp
  #
  # @brief ꤷ֥ȤNamingService饢Х
  # 
  # ꤷ֥Ȥ NamingService 饢Хɤ롣
  # 
  # @param self
  # @param name Хоݥ֥
  #
  # @else
  #
  # @endif
  def unbindObject(self, name):
    self._rtcout.RTC_TRACE("NamingManager::unbindObject(%s)", name)
    guard = OpenRTM_aist.ScopedLock(self._namesMutex)
    for i in range(len(self._names)):
      if self._names[i].ns:
        self._names[i].ns.unbindObject(name)
    self.unregisterCompName(name)
    self.unregisterMgrName(name)


  ##
  # @if jp
  #
  # @brief ƤΥ֥ȤNamingService饢Х
  # 
  # ƤΥ֥Ȥ CORBA NamingService 饢Хɤ롣
  # 
  # @param self
  # 
  # @else
  #
  # @endif
  def unbindAll(self):
    self._rtcout.RTC_TRACE("NamingManager::unbindAll(): %d names.", len(self._compNames))

    guard = OpenRTM_aist.ScopedLock(self._compNamesMutex)
    for i in range(len(self._compNames)):
      self.unbindObject(self._compNames[i].name)

    guard = OpenRTM_aist.ScopedLock(self._mgrNamesMutex)
    for i in range(len(self._mgrNames)):
      self.unbindObject(self._mgrNames[i].name)


  ##
  # @if jp
  #
  # @brief ХɤƤƤΥ֥Ȥ
  # 
  # ХɤƤƤΥ֥Ȥ 롣
  # 
  # @param self
  #
  # @return ХɺѤߥ֥ ꥹ
  # 
  # @else
  #
  # @endif
  def getObjects(self):
    comps = []
    guard = OpenRTM_aist.ScopedLock(self._compNamesMutex)
    for i in range(len(self._compNames)):
      comps.append(self._compNames[i].rtobj)
    return comps


  ##
  # @if jp
  #
  # @brief NameServer ѥ֥Ȥ
  # 
  # ꤷNameServer ѥ֥Ȥ롣
  #
  # @param self
  # @param method NamingService 
  # @param name_server NameServer ̾
  # 
  # @return  NameServer ֥
  # 
  # @else
  #
  # @endif
  def createNamingObj(self, method, name_server):
    mth = method
    if mth == "corba":
      try:
        name = OpenRTM_aist.NamingOnCorba(self._manager.getORB(),name_server)
        if name is None:
          return None
        self._rtcout.RTC_INFO("NameServer connection succeeded: %s/%s",
                              (method, name_server))
        return name
      except:
        self._rtcout.RTC_INFO("NameServer connection failed: %s/%s",
                              (method, name_server))
        return None

    return None


  ##
  # @if jp
  #
  # @brief ѤߥݡͥȤ NameServer Ͽ
  # 
  # ѤߥݡͥȤꤷ NameServer Ͽ롣
  #
  # @param self
  # @param ns Ͽо NameServer
  # 
  # @else
  #
  # @endif
  def bindCompsTo(self, ns):
    for i in range(len(self._compNames)):
      ns.bindObject(self._compNames[i].name, self._compNames[i].rtobj)


  ##
  # @if jp
  #
  # @brief NameServer Ͽ륳ݡͥȤ
  # 
  # NameServer Ͽ륳ݡͥȤꤹ롣
  #
  # @param self
  # @param name ݡͥȤϿ̾
  # @param rtobj Ͽоݥ֥
  # 
  # @else
  #
  # @endif
  def registerCompName(self, name, rtobj):
    for i in range(len(self._compNames)):
      if self._compNames[i].name == name:
        self._compNames[i].rtobj = rtobj
        return

    self._compNames.append(self.Comps(name, rtobj))
    return


  def registerMgrName(self, name, mgr):
    for i in range(len(self._mgrNames)):
      if self._mgrNames[i].name == name:
        self._mgrNames[i].mgr = mgr
        return

    self._mgrNames.append(self.Mgr(name, mgr))
    return


  ##
  # @if jp
  #
  # @brief NameServer Ͽ륳ݡͥȤ
  # 
  # NameServer Ͽ륳ݡͥȤ롣
  #
  # @param self
  # @param name оݥݡͥȤ̾
  # 
  # @else
  #
  # @endif
  def unregisterCompName(self, name):
    len_ = len(self._compNames)
    for i in range(len_):
      idx = (len_-1) - i
      if self._compNames[idx].name == name:
        del self._compNames[idx]
        return
    return
    

  def unregisterMgrName(self, name):
    len_ = len(self._mgrNames)
    for i in range(len_):
      idx = (len_ -1) - i
      if self._mgrNames[idx].name == name:
        del self._mgrNames[idx]
        return
    return


  # Name Servers' method/name and object
  ##
  # @if jp
  # @class Names
  # @brief NameServer ѥ饹
  # @else
  #
  # @endif
  class Names:
    def __init__(self, meth, name, naming):
      self.method = meth
      self.nsname = name
      self.ns     = naming


  # Components' name and object
  ##
  # @if jp
  # @class Comps
  # @brief ݡͥȴѥ饹
  # @else
  #
  # @endif
  class Comps:
    def __init__(self, n, obj):
      self.name = n
      self.rtobj = obj


  class Mgr:
    def __init__(self, n, obj):
      self.name = n
      self.mgr = obj
