#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file PortBase.py
# @brief RTC's Port base class
# @date $Date: 2007/09/18 $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2006-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.


import threading
import copy

import OpenRTM_aist
import RTC, RTC__POA



##
# @if jp
# @class PortBase
# @brief Port δ쥯饹
#
# RTC::Port δȤʤ륯饹
# RTC::Port Ϥۤ UML Port γǰѾƤꡢۤƱΤΤȤߤʤ
# ȤǤ롣RT ݡͥȤΥ󥻥ץȤˤƤϡ
# Port ϥݡͥȤ°ݡͥȤ¾ΥݡͥȤߺ
# ԤǤꡢ̾ĤΥ󥿡եȴϢդ롣
# ݡͥȤ Port ̤ƳФ󥿡ե󶡤ޤ׵
# 뤳ȤǤPortϤ³ô
# <p>
# Port ζݥ饹ϡ̾ RT ݡͥȥ󥹥Ʊ
# 졢󶡡׵ᥤ󥿡եϿ塢RT ݡͥȤ
# Ͽ졢饢ǽ Port ȤƵǽ뤳ȤꤷƤ롣
# <p>
# RTC::Port  CORBA 󥿡եȤưʲΥڥ졼󶡤롣
#
# - get_port_profile()
# - get_connector_profiles()
# - get_connector_profile()
# - connect()
# - notify_connect()
# - disconnect()
# - notify_disconnect()
# - disconnect_all()
#
# Υ饹ǤϡΥڥ졼μ󶡤롣
# <p>
# Υڥ졼Τget_port_profile(), get_connector_profiles(),
# get_connector_profile(), connect(), disconnect(), disconnect_all() ϡ
# ֥饹ˤä˿񤤤ѹɬפʤᡢС饤
# 뤳ȤϿ侩ʤ
# <p>
# notify_connect(), notify_disconnect() ˤĤƤϡ֥饹󶡡׵
# 륤󥿡եμ˱ơ񤤤ѹɬפ
# ⤷ʤľܥС饤ɤ뤳ȤϿ侩줺
# Ҥ notify_connect(), notify_disconnect() ιˤƤҤ٤̤
# δؿ˴Ϣ ؿ򥪡С饤ɤ뤳Ȥˤ꿶񤤤ѹ
# Ȥ侩롣
#
# @since 0.4.0
#
# @else
#
#
# @endif
class PortBase(RTC__POA.PortService):
  """
  """



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # PortBase Υ󥹥ȥ饯 Port ̾ name ˼Ԥ
  # ƱˡʬȤ CORBA Object ȤƳȤ PortProfile
  #  port_ref ˼ȤΥ֥ȥե󥹤Ǽ롣
  #
  # @param self
  # @param name Port ̾(ǥե:None)
  #
  # @else
  #
  # @brief Constructor
  #
  # The constructor of the ProtBase class is given the name of this Port
  # and initialized. At the same time, the PortBase activates itself
  # as CORBA object and stores its object reference to the PortProfile's 
  # port_ref member.
  #
  # @param name The name of Port 
  #
  # @endif
  def __init__(self, name=None):
    self._profile = RTC.PortProfile("", [], RTC.PortService._nil, [], RTC.RTObject._nil,[])
    
    if name is None:
      self._profile.name = ""
    else:
      self._profile.name = name
      
    self._objref = self._this()
    self._profile.port_ref = self._objref
    self._profile.owner = RTC.RTObject._nil
    self._profile_mutex = threading.RLock()
    self._rtcout = OpenRTM_aist.Manager.instance().getLogbuf(name)


  def __del__(self):
    self._rtcout.RTC_TRACE("PortBase.__del__()")
    try:
      mgr = OpenRTM_aist.Manager.instance().getPOA()
      oid = mgr.servant_to_id(self)
      mgr.deactivate_object(oid)
    except:
      self._rtcout.RTC_WARN("Unknown exception caught.")
    

  ##
  # @if jp
  #
  # @brief [CORBA interface] PortProfile
  #
  # PortݻPortProfile֤
  # PortProfile ¤ΤϰʲΥСġ
  #
  # - name              [string ] Port ̾
  # - interfaces        [PortInterfaceProfileList ] Port ݻ
  #                     PortInterfaceProfile Υ
  # - port_ref          [Port Object ] Port ȤΥ֥ȥե
  # - connector_profile [ConnectorProfileList ] Port ݻ
  #                     ConnectorProfile Υ
  # - owner             [RTObject Object ]  Port ͭ
  #                     RTObjectΥե
  # - properties        [NVList ] ¾Υץѥƥ
  #
  # @param self
  #
  # @return PortProfile
  #
  # @else
  #
  # @brief [CORBA interface] Get the PortProfile of the Port
  #
  # This operation returns the PortProfile of the Port.
  # PortProfile struct has the following members,
  #
  # - name              [string ] The name of the Port.
  # - interfaces        [PortInterfaceProfileList ] The sequence of 
  #                     PortInterfaceProfile owned by the Port
  # - port_ref          [Port Object] The object reference of the Port.
  # - connector_profile [ConnectorProfileList ] The sequence of 
  #                     ConnectorProfile owned by the Port.
  # - owner             [RTObject Object] The object reference of 
  #                     RTObject that is owner of the Port.
  # - properties        [NVList] The other properties.
  #
  # @return the PortProfile of the Port
  #
  # @endif
  def get_port_profile(self):
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)

    prof = RTC.PortProfile(self._profile.name,
                           self._profile.interfaces,
                           self._profile.port_ref,
                           self._profile.connector_profiles,
                           self._profile.owner,
                           self._profile.properties)

    return prof


  def getPortProfile(self):
    self._rtcout.RTC_TRACE("getPortProfile()")
    return self._profile


  ##
  # @if jp
  #
  # @brief [CORBA interface] ConnectorProfileList
  #
  # Portݻ ConnectorProfile  sequence ֤
  # ConnectorProfile  Port ֤³ץեݻ빽¤ΤǤꡢ
  # ³Port֤Ǿ򴹤ԤϢ뤹٤Ƥ Port Ʊͤ
  # ݻ롣
  # ConnectorProfile ϰʲΥСݻƤ롣
  #
  # - name         [string ] Υͥ̾
  # - connector_id [string ] ˡID
  # - ports        [Port sequnce] Υͥ˴Ϣ Port Υ֥
  #                ե󥹤Υ󥹡
  # - properties   [NVList ] ¾Υץѥƥ
  #
  # @param self
  #
  # @return  Port ݻ ConnectorProfile
  #
  # @else
  #
  # @brief [CORBA interface] Get the ConnectorProfileList of the Port
  #
  # This operation returns a list of the ConnectorProfiles of the Port.
  # ConnectorProfile includes the connection information that describes 
  # relation between (among) Ports, and Ports exchange the ConnectionProfile
  # on connection process and hold the same information in each Port.
  # ConnectionProfile has the following members,
  #
  # - name         [string] The name of the connection.
  # - connector_id [string] Unique identifier.
  # - ports        [Port sequnce] The sequence of Port's object reference
  #                that are related the connection.
  # - properties   [NVList] The other properties.
  #
  # @return the ConnectorProfileList of the Port
  #
  # @endif
  # virtual ConnectorProfileList* get_connector_profiles()
  def get_connector_profiles(self):
    self._rtcout.RTC_TRACE("get_connector_profiles()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    return self._profile.connector_profiles


  ##
  # @if jp
  #
  # @brief [CORBA interface] ConnectorProfile 
  #
  # connector_id ǻꤵ줿 ConnectorProfile ֤
  # ꤷ connector_id  ConnectorProfile ݻƤʤϡ
  #  ConnectorProfile ֤
  #
  # @param self
  # @param connector_id ConnectorProfile  ID
  #
  # @return connector_id ǻꤵ줿 ConnectorProfile
  #
  # @else
  #
  # @brief [CORBA interface] Get the ConnectorProfile
  #
  # This operation returns the ConnectorProfiles specified connector_id.
  #
  # @param connector_id ID of the ConnectorProfile
  #
  # @return the ConnectorProfile identified by the connector_id
  #
  # @endif
  def get_connector_profile(self, connector_id):
    self._rtcout.RTC_TRACE("get_connector_profile(%s)", connector_id)
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                            self.find_conn_id(connector_id))
    if index < 0:
      conn_prof = RTC.ConnectorProfile("","",[],[])
      return conn_prof

    conn_prof = RTC.ConnectorProfile(self._profile.connector_profiles[index].name,
                                     self._profile.connector_profiles[index].connector_id,
                                     self._profile.connector_profiles[index].ports,
                                     self._profile.connector_profiles[index].properties)
    return conn_prof


  ##
  # @if jp
  #
  # @brief [CORBA interface] Port ³Ԥ
  #
  # Ϳ줿 ConnectoionProfile ξˡPort֤³Ω롣
  # ץꥱץ¦ϡĤΥݡͥȤʣ
  # Port ³硢Ŭڤͤ򥻥åȤ ConnectorProfile 
  # connect() ΰȤͿƥ뤹뤳ȤˤꡢϢ Port 
  # ³Ω롣
  #
  # connect() Ϳ ConnectorProfile ΥСΤname, ports, 
  # properties СФƥǡ򥻥åȤʤФʤʤ
  #
  # OutPort ¦ connect() ǤϰʲΥ󥹤ǽԤ롣
  #
  # 1. OutPort ˴Ϣ connector ӥå
  #
  # 2. InPort˴Ϣ connector μ
  #  - ConnectorProfile::properties["dataport.corba_any.inport_ref"]
  #    OutPortAny Υ֥ȥե󥹤ꤵƤ硢
  #    ե󥹤Consumer֥Ȥ˥åȤ롣
  #    ե󥹤åȤƤʤ̵뤷Ʒ³
  #    (OutPortconnect() ƤӽФΥȥݥȤξϡ
  #    InPortΥ֥ȥե󥹤ϥåȤƤʤϤǤ롣)
  #
  # 3. PortBase::connect() 򥳡
  #    Port³δܽԤ롣
  #
  # 4. 嵭2.InPortΥե󥹤ǤʤСInPort
  #    Ϣ connector 롣
  #
  # 5. ConnectorProfile::properties Ϳ줿󤫤顢
  #    OutPort¦νԤ
  #
  # - [dataport.interface_type]<BR>
  # -- CORBA_Any ξ: 
  #    InPortAny ̤ƥǡ򴹤롣
  #    ConnectorProfile::properties["dataport.corba_any.inport_ref"]
  #    InPortAny Υ֥ȥե󥹤򥻥åȤ롣<BR>
  #
  # - [dataport.dataflow_type]<BR>
  # -- Pushξ: Subscriber롣SubscriberΥפϡ
  #    dataport.subscription_type ꤵƤ롣<BR>
  # -- Pullξ: InPort¦ǡPullǼ뤿ᡢ
  #    ä˲⤹ɬפ̵
  #
  # - [dataport.subscription_type]<BR>
  # -- Onceξ: SubscriberOnce롣<BR>
  # -- Newξ: SubscriberNew롣<BR>
  # -- Periodicξ: SubscriberPeriodic롣
  #
  # - [dataport.push_rate]<BR>
  # -- dataport.subscription_type=Periodicξꤹ롣
  #
  # 6. 嵭νΤĤǤ⥨顼ǤС顼꥿󤹤롣
  #    ˽Ԥ줿 RTC::RTC_OK ǥ꥿󤹤롣
  #  
  # @param self
  # @param connector_profile ConnectorProfile
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [CORBA interface] Connect the Port
  #
  # This operation establishes connection according to the given 
  # ConnectionProfile inforamtion. 
  # Application programs, which is going to establish the connection 
  # among Ports owned by RT-Components, have to set valid values to the 
  # ConnectorProfile and give it to the argument of connect() operation.
  # 
  # name, ports, properties members of ConnectorProfile should be set
  # valid values before giving to the argument of connect() operation.
  #
  # @param connector_profile The ConnectorProfile.
  #
  # @return ReturnCode_t The return code of this operation.
  #
  # @endif
  # virtual ReturnCode_t connect(ConnectorProfile& connector_profile)
  def connect(self, connector_profile):
    self._rtcout.RTC_TRACE("connect()")
    if self.isEmptyId(connector_profile):
      guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
      self.setUUID(connector_profile)
      assert(not self.isExistingConnId(connector_profile.connector_id))
      del guard
    else:
      guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
      if self.isExistingConnId(connector_profile.connector_id):
        self._rtcout.RTC_ERROR("Connection already exists.")
        return (RTC.PRECONDITION_NOT_MET,connector_profile)
      del guard

    try:
      retval,connector_profile = connector_profile.ports[0].notify_connect(connector_profile)
      if retval != RTC.RTC_OK:
        self._rtcout.RTC_ERROR("Connection failed. cleanup.")
        self.disconnect(connector_profile.connector_id)
    
      return (retval, connector_profile)
      #return connector_profile.ports[0].notify_connect(connector_profile)
    except:
      return (RTC.BAD_PARAMETER, connector_profile)

    return (RTC.RTC_ERROR, connector_profile)


  ##
  # @if jp
  #
  # @brief [CORBA interface] Port ³ΤԤ
  #
  # Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
  # ƤФ륪ڥ졼Ǥ롣
  # ConnectorProfile ˤ³о Port ΥꥹȾݻƤ롣Port 
  # ConnectorProfile ݻȤȤˡꥹμ Port  notify_connect 
  # ƤӽФơݡȤ򥳥ͥɲä塢ConnectorProfile 
  # ƤӤ Port ꤷƤӤ֤Τ褦 ConnectorProfile
  # Ѥ³ΤãƤ
  #
  # @param self
  # @param connector_profile ConnectorProfile
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [CORBA interface] Notify the Ports connection
  #
  # This operation is invoked between Ports internally when the connection
  # is established.
  # This operation notifies this PortService of the connection between its 
  # corresponding port and the other ports and propagates the given 
  # ConnectionProfile.
  # A ConnectorProfile has a sequence of port references. This PortService 
  # stores the ConnectorProfile and invokes the notify_connect operation of 
  # the next PortService in the sequence. As ports are added to the 
  # connector, PortService references are added to the ConnectorProfile and
  # provided to the caller. In this way, notification of connection is 
  # propagated with the ConnectorProfile.
  #
  # @param connector_profile The ConnectorProfile.
  #
  # @return ReturnCode_t The return code of this operation.
  #
  # @endif
  def notify_connect(self, connector_profile):
    self._rtcout.RTC_TRACE("notify_connect()")

    # publish owned interface information to the ConnectorProfile
    retval = [RTC.RTC_OK for i in range(3)]

    retval[0] = self.publishInterfaces(connector_profile)
    if retval[0] != RTC.RTC_OK:
      self._rtcout.RTC_ERROR("publishInterfaces() in notify_connect() failed.")

    # call notify_connect() of the next Port
    retval[1], connector_profile = self.connectNext(connector_profile)
    if retval[1] != RTC.RTC_OK:
      self._rtcout.RTC_ERROR("connectNext() in notify_connect() failed.")

    # subscribe interface from the ConnectorProfile's information
    retval[2] = self.subscribeInterfaces(connector_profile)
    if retval[2] != RTC.RTC_OK:
      self._rtcout.RTC_ERROR("subscribeInterfaces() in notify_connect() failed.")
      #self.notify_disconnect(connector_profile.connector_id)

    self._rtcout.RTC_PARANOID("%d connectors are existing",
                              len(self._profile.connector_profiles))

    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    # update ConnectorProfile
    index = self.findConnProfileIndex(connector_profile.connector_id)
    if index < 0:
      OpenRTM_aist.CORBA_SeqUtil.push_back(self._profile.connector_profiles,
                                           connector_profile)
      self._rtcout.RTC_PARANOID("New connector_id. Push backed.")

    else:
      self._profile.connector_profiles[index] = connector_profile
      self._rtcout.RTC_PARANOID("Existing connector_id. Updated.")

    for ret in retval:
      if ret != RTC.RTC_OK:
        return (ret, connector_profile)

    return (RTC.RTC_OK, connector_profile)


  ##
  # @if jp
  #
  # @brief [CORBA interface] Port ³
  #
  # Υڥ졼³Ω³ФͿ connector_id 
  # бԥ Port Ȥ³롣
  # Port  ConnectorProfile ΥݡȥꥹȤ˴ޤޤ룱ĤΥݡȤ
  # notify_disconnect ƤӤ³Τ notify_disconnect ˤä
  # ¹Ԥ롣
  #
  # @param self
  # @param connector_id ConnectorProfile  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [CORBA interface] Connect the Port
  #
  # This operation destroys connection between this port and the peer port
  # according to given id that is given when the connection established.
  # This port invokes the notify_disconnect operation of one of the ports 
  # included in the sequence of the ConnectorProfile stored when the 
  # connection was established. The notification of disconnection is 
  # propagated by the notify_disconnect operation.
  #
  # @param connector_id The ID of the ConnectorProfile.
  #
  # @return ReturnCode_t The return code of this operation.
  #
  # @endif
  # virtual ReturnCode_t disconnect(const char* connector_id)
  def disconnect(self, connector_id):
    self._rtcout.RTC_TRACE("disconnect(%s)", connector_id)

    index = self.findConnProfileIndex(connector_id)

    if index < 0:
      self._rtcout.RTC_ERROR("Invalid connector id: %s", connector_id)
      return RTC.BAD_PARAMETER

    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    prof = self._profile.connector_profiles[index]
    del guard
    
    if len(prof.ports) < 1:
      self._rtcout.RTC_FATAL("ConnectorProfile has empty port list.")
      return RTC.PRECONDITION_NOT_MET

    for i in range(len(prof.ports)):
      p = prof.ports[i]
      try:
        return p.notify_disconnect(connector_id)
      except:
        self._rtcout.RTC_WARN("Unknown exception caught.")
        continue

    self._rtcout.RTC_ERROR("notify_disconnect() for all ports failed.")
    return RTC.RTC_ERROR


  ##
  # @if jp
  #
  # @brief [CORBA interface] Port ³ΤԤ
  #
  # Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
  # ƤФ륪ڥ졼Ǥ롣
  # Υڥ졼ϡ Port ³Ƥ¾ Port ³
  # Τ롣³оݤ Port IDˤäƻꤵ롣Port 
  # ConnectorProfile Υݡȥꥹμ Port  notify_disconnect Ƥ
  # ФݡȤ³ ConnectorProfile 鳺 Port 
  # 󤬺롣Τ褦 notify_disconnect Ѥ³Τ
  # ãƤ
  #
  # @param self
  # @param connector_id ConnectorProfile  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [CORBA interface] Notify the Ports disconnection
  #
  # This operation is invoked between Ports internally when the connection
  # is destroied.
  # This operation notifies a PortService of a disconnection between its 
  # corresponding port and the other ports. The disconnected connector is 
  # identified by the given ID, which was given when the connection was 
  # established.
  # This port invokes the notify_disconnect operation of the next PortService
  # in the sequence of the ConnectorProfile that was stored when the 
  # connection was established. As ports are disconnected, PortService 
  # references are removed from the ConnectorProfile. In this way, 
  # the notification of disconnection is propagated by the notify_disconnect
  # operation.
  #
  # @param connector_id The ID of the ConnectorProfile.
  #
  # @return ReturnCode_t The return code of this operation.
  #
  # @endif
  # virtual ReturnCode_t notify_disconnect(const char* connector_id)
  def notify_disconnect(self, connector_id):
    self._rtcout.RTC_TRACE("notify_disconnect(%s)", connector_id)

    # The Port of which the reference is stored in the beginning of
    # connectorProfile's PortServiceList is master Port.
    # The master Port has the responsibility of disconnecting all Ports.
    # The slave Ports have only responsibility of deleting its own
    # ConnectorProfile.

    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)

    index = self.findConnProfileIndex(connector_id)

    if index < 0:
      self._rtcout.RTC_ERROR("Invalid connector id: %s", connector_id)
      return RTC.BAD_PARAMETER

    prof = RTC.ConnectorProfile(self._profile.connector_profiles[index].name,
                                self._profile.connector_profiles[index].connector_id,
                                self._profile.connector_profiles[index].ports,
                                self._profile.connector_profiles[index].properties)

    retval = self.disconnectNext(prof)
    self.unsubscribeInterfaces(prof)

    OpenRTM_aist.CORBA_SeqUtil.erase(self._profile.connector_profiles, index)
    
    return retval


  ##
  # @if jp
  #
  # @brief [CORBA interface] Port ³
  #
  # Υڥ졼Ϥ Port ˴ϢƤ³롣
  #
  # @param self
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [CORBA interface] Connect the Port
  #
  # This operation destroys all connection channels owned by the Port.
  #
  # @return ReturnCode_t The return code of this operation.
  #
  # @endif
  # virtual ReturnCode_t disconnect_all()
  def disconnect_all(self):
    self._rtcout.RTC_TRACE("disconnect_all()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    plist = copy.deepcopy(self._profile.connector_profiles)
    del guard
    
    retcode = RTC.RTC_OK
    len_ = len(plist)
    self._rtcout.RTC_DEBUG("disconnecting %d connections.", len_)

    # disconnect all connections
    # Call disconnect() for each ConnectorProfile.
    for i in range(len_):
      tmpret = self.disconnect(plist[i].connector_id)
      if tmpret != RTC.RTC_OK:
        retcode = tmpret

    return retcode


  #============================================================
  # Local operations
  #============================================================

  ##
  # @if jp
  # @brief Port ̾ꤹ
  #
  # Port ̾ꤹ롣̾ Port ݻ PortProfile.name
  # ȿǤ롣
  #
  # @param self
  # @param name Port ̾
  #
  # @else
  # @brief Set the name of this Port
  #
  # This operation sets the name of this Port. The given Port's name is
  # applied to Port's PortProfile.name.
  #
  # @param name The name of this Port.
  #
  # @endif
  # void setName(const char* name);
  def setName(self, name):
    self._rtcout.RTC_TRACE("setName(%s)", name)
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    self._profile.name = name


  ##
  # @if jp
  # @brief PortProfile
  #
  # Portݻ PortProfile  const Ȥ֤
  #
  # @param self
  #
  # @return  Port  PortProfile
  #
  # @else
  # @brief Get the PortProfile of the Port
  #
  # This operation returns const reference of the PortProfile.
  #
  # @return the PortProfile of the Port
  #
  # @endif
  # const PortProfile& getProfile() const;
  def getProfile(self):
    self._rtcout.RTC_TRACE("getProfile()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    return self._profile


  ##
  # @if jp
  #
  # @brief Port Υ֥ȻȤꤹ
  #
  # Υڥ졼 Port  PortProfile ˤ Port Ȥ
  # ֥ȻȤꤹ롣
  #
  # @param self
  # @param port_ref  Port Υ֥Ȼ
  #
  # @else
  #
  # @brief Set the object reference of this Port
  #
  # This operation sets the object reference itself
  # to the Port's PortProfile.
  #
  # @param The object reference of this Port.
  #
  # @endif
  # void setPortRef(PortService_ptr port_ref);
  def setPortRef(self, port_ref):
    self._rtcout.RTC_TRACE("setPortRef()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    self._profile.port_ref = port_ref


  ##
  # @if jp
  #
  # @brief Port Υ֥ȻȤ
  #
  # Υڥ졼 Port  PortProfile ݻƤ
  #  Port ȤΥ֥ȻȤ롣
  #
  # @param self
  #
  # @return  Port Υ֥Ȼ
  #
  # @else
  #
  # @brief Get the object reference of this Port
  #
  # This operation returns the object reference
  # that is stored in the Port's PortProfile.
  #
  # @return The object reference of this Port.
  #
  # @endif
  # PortService_ptr getPortRef();
  def getPortRef(self):
    self._rtcout.RTC_TRACE("getPortRef()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    return self._profile.port_ref


  ##
  # @if jp
  #
  # @brief Port  owner  RTObject ꤹ
  #
  # Υڥ졼 Port  PortProfile.owner ꤹ롣
  #
  # @param self
  # @param owner  Port ͭ RTObject λ
  #
  # @else
  #
  # @brief Set the owner RTObject of the Port
  #
  # This operation sets the owner RTObject of this Port.
  #
  # @param owner The owner RTObject's reference of this Port
  #
  # @endif
  # void setOwner(RTObject_ptr owner);
  def setOwner(self, owner):
    self._rtcout.RTC_TRACE("setOwner()")
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)
    self._profile.owner = owner


  ##
  # @if jp
  #
  # @brief Interface (֥饹)
  #
  # Υڥ졼ϡnotify_connect() 󥹤λϤ˥
  # ؿǤ롣
  # notify_connect() Ǥϡ
  #
  # - publishInterfaces()
  # - connectNext()
  # - subscribeInterfaces()
  # - updateConnectorProfile()
  #
  # ν protected ؿ뤵³Ԥ롣
  # <br>
  #  Port ǤϤΥڥ졼򥪡С饤ɤȤ
  # Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
  # ǤСRteurnCode_t Υ顼ɤ֤
  # ̾ publishInterafaces() ˤƤϡ Port °
  # 󥿡ե˴ؤ ConnectorProfile ФŬڤꤷ
  # ¾ Port ΤʤФʤʤ
  # <br>
  # ޤδؿ뤵ʳǤϡ¾ Port  Interface ˴ؤ
  # Ϥ٤ƴޤޤƤʤΤǡ¾ Port  Interface 
  #  subscribeInterfaces() ǹԤ٤Ǥ롣
  # <br>
  # Υڥ졼ϡ connector_id ФƤ³
  # ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣<BR>
  # ֥饹Ǥμ
  #
  # @param self
  # @param connector_profile ³˴ؤץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Publish interface information
  #
  # This operation is pure virutal method that would be called at the
  # beginning of the notify_connect() process sequence.
  # In the notify_connect(), the following methods would be called in order.
  #
  # - publishInterfaces()
  # - connectNext()
  # - subscribeInterfaces()
  # - updateConnectorProfile() 
  #
  # In the concrete Port, this method should be overridden. This method
  # processes the given ConnectorProfile argument and if the given parameter
  # is invalid, it would return error code of ReturnCode_t.
  # Usually, publishInterfaces() method should set interfaces information
  # owned by this Port, and publish it to the other Ports.
  # <br>
  # When this method is called, other Ports' interfaces information may not
  # be completed. Therefore, the process to obtain other Port's interfaces
  # information should be done in the subscribeInterfaces() method.
  # <br>
  # This operation should create the new connection for the new
  # connector_id, and should update the connection for the existing
  # connection_id.
  #
  # @param connector_profile The connection profile information
  # @return The return code of ReturnCode_t type.
  #
  #@endif
  def publishInterfaces(self, connector_profile):
    pass


  ##
  # @if jp
  #
  # @brief  Port Ф notify_connect() 򥳡뤹
  #
  # ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
  # ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
  # notify_connect() 򥳡뤹롣
  #
  # @param self
  # @param connector_profile ³˴ؤץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Call notify_connect() of the next Port
  #
  # This operation calls the notify_connect() of the next Port's 
  # that stored in ConnectorProfile's port_ref sequence.
  #
  # @param connector_profile The connection profile information
  #
  # @return The return code of ReturnCode_t type.
  #
  # @endif
  # virtual ReturnCode_t connectNext(ConnectorProfile& connector_profile);
  def connectNext(self, connector_profile):
    index = OpenRTM_aist.CORBA_SeqUtil.find(connector_profile.ports,
                                            self.find_port_ref(self._profile.port_ref))
    if index < 0:
      return RTC.BAD_PARAMETER, connector_profile

    index += 1
    if index < len(connector_profile.ports):
      p = connector_profile.ports[index]
      return p.notify_connect(connector_profile)

    return (RTC.RTC_OK, connector_profile)


  ##
  # @if jp
  #
  # @brief  Port Ф notify_disconnect() 򥳡뤹
  #
  # ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
  # ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
  # notify_disconnect() 򥳡뤹롣
  #
  # @param self
  # @param connector_profile ³˴ؤץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Call notify_disconnect() of the next Port
  #
  # This operation calls the notify_disconnect() of the next Port's 
  # that stored in ConnectorProfile's port_ref sequence.
  #
  # @param connector_profile The connection profile information
  #
  # @return The return code of ReturnCode_t type.
  #
  # @endif
  # virtual ReturnCode_t disconnectNext(ConnectorProfile& connector_profile);
  def disconnectNext(self, connector_profile):
    index = OpenRTM_aist.CORBA_SeqUtil.find(connector_profile.ports,
                                            self.find_port_ref(self._profile.port_ref))
    if index < 0:
      return RTC.BAD_PARAMETER

    index += 1

    while index < len(connector_profile.ports):
      p = connector_profile.ports[index]
      index += 1
      try:
        return p.notify_disconnect(connector_profile.connector_id)
      except:
        self._rtcout.RTC_WARN("Unknown exception caught.")
        continue

    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief Interface (֥饹)
  #
  # Υڥ졼ϡnotify_connect() 󥹤֤˥
  # ؿǤ롣
  # notify_connect() Ǥϡ
  #
  #  - publishInterfaces()
  #  - connectNext()
  #  - subscribeInterfaces()
  #  - updateConnectorProfile()
  #
  # ν protected ؿ뤵³Ԥ롣
  # <br>
  #  Port ǤϤΥڥ졼򥪡С饤ɤȤ
  # Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
  # ǤСRteurnCode_t Υ顼ɤ֤
  #  ConnectorProfile ˤ¾ Port  Interface ˴ؤ
  # ƴޤޤƤ롣
  # ̾ subscribeInterafaces() ˤƤϡ Port Ѥ
  # Interface ˴ؤ׵¦Υ󥿡եФ
  # ꤷʤФʤʤ
  # <br>
  # Υڥ졼ϡ connector_id ФƤ³
  # ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣<BR>
  # ֥饹Ǥμ
  #
  # @param self
  # @param connector_profile ³˴ؤץե
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief Publish interface information
  #
  # This operation is pure virutal method that would be called at the
  # mid-flow of the notify_connect() process sequence.
  # In the notify_connect(), the following methods would be called in order.
  #
  #  - publishInterfaces()
  #  - connectNext()
  #  - subscribeInterfaces()
  #  - updateConnectorProfile()
  #
  # In the concrete Port, this method should be overridden. This method
  # processes the given ConnectorProfile argument and if the given parameter
  # is invalid, it would return error code of ReturnCode_t.
  # The given argument ConnectorProfile includes all the interfaces
  # information in it.
  # Usually, subscribeInterafaces() method obtains information of interfaces
  # from ConnectorProfile, and should set it to the interfaces that require
  # them.
  # <br>
  # This operation should create the new connection for the new
  # connector_id, and should update the connection for the existing
  # connection_id.
  #
  # @param connector_profile The connection profile information
  #
  # @return The return code of ReturnCode_t type.
  #
  #@endif
  def subscribeInterfaces(self, connector_profile):
    pass


  ##
  # @if jp
  #
  # @brief Interface ³(֥饹)
  #
  # Υڥ졼ϡnotify_disconnect() 󥹤ν˥
  # ؿǤ롣
  # notify_disconnect() Ǥϡ
  #  - disconnectNext()
  #  - unsubscribeInterfaces()
  #  - eraseConnectorProfile()
  # ν protected ؿ뤵³Ԥ롣
  # <br>
  #  Port ǤϤΥڥ졼򥪡С饤ɤȤ
  # Ϳ줿 ConnectorProfile ˽³Ԥ<BR>
  # ֥饹Ǥμ
  #
  # @param self
  # @param connector_profile ³˴ؤץե
  #
  # @else
  #
  # @brief Disconnect interface connection
  #
  # This operation is pure virutal method that would be called at the
  # end of the notify_disconnect() process sequence.
  # In the notify_disconnect(), the following methods would be called.
  #  - disconnectNext()
  #  - unsubscribeInterfaces()
  #  - eraseConnectorProfile() 
  # <br>
  # In the concrete Port, this method should be overridden. This method
  # processes the given ConnectorProfile argument and disconnect interface
  # connection.
  #
  # @param connector_profile The connection profile information
  #
  # @endif
  def unsubscribeInterfaces(self, connector_profile):
    pass


  ##
  # @if jp
  #
  # @brief ConnectorProfile  connector_id եɤɤȽ
  #
  # ꤵ줿 ConnectorProfile  connector_id Ǥ뤫ɤȽ
  # Ԥ
  #
  # @param self
  # @param connector_profile Ƚоݥͥץե
  #
  # @return Ϳ줿 ConnectorProfile  connector_id ǤС
  #         trueǤʤ false ֤
  #
  # @else
  #
  # @brief Whether connector_id of ConnectorProfile is empty
  #
  # @return If the given ConnectorProfile's connector_id is empty string,
  #         it returns true.
  #
  # @endif
  # bool isEmptyId(const ConnectorProfile& connector_profile) const;
  def isEmptyId(self, connector_profile):
    return connector_profile.connector_id == ""


  ##
  # @if jp
  #
  # @brief UUID
  #
  # Υڥ졼 UUID 롣
  #
  # @param self
  #
  # @return uuid
  #
  # @else
  #
  # @brief Get the UUID
  #
  # This operation generates UUID.
  #
  # @return uuid
  #
  # @endif
  # const std::string getUUID() const;
  def getUUID(self):
    return str(OpenRTM_aist.uuid1())


  ##
  # @if jp
  #
  # @brief UUID ConnectorProfile ˥åȤ
  #
  # Υڥ졼 UUID ConnectorProfile ˥åȤ롣
  #
  # @param self
  # @param connector_profile connector_id 򥻥åȤ ConnectorProfile
  #
  # @else
  #
  # @brief Create and set the UUID to the ConnectorProfile
  #
  # This operation generates and set UUID to the ConnectorProfile.
  #
  # @param connector_profile ConnectorProfile to be set connector_id
  #
  # @endif
  # void setUUID(ConnectorProfile& connector_profile) const;
  def setUUID(self, connector_profile):
    connector_profile.connector_id = self.getUUID()
    assert(connector_profile.connector_id != "")


  ##
  # @if jp
  #
  # @brief id ¸ ConnectorProfile ΤΤɤȽꤹ
  #
  # Υڥ졼Ϳ줿 ID ¸ ConnectorProfile Υꥹ
  # ¸ߤ뤫ɤȽꤹ롣
  #
  # @param self
  # @param id_ Ƚꤹ connector_id
  #
  # @return id ¸Ƚ
  #
  # @else
  #
  # @brief Whether the given id exists in stored ConnectorProfiles
  #
  # This operation returns boolean whether the given id exists in 
  # the Port's ConnectorProfiles.
  #
  # @param id connector_id to be find in Port's ConnectorProfiles
  #
  # @endif
  # bool isExistingConnId(const char* id);
  def isExistingConnId(self, id_):
    return OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                           self.find_conn_id(id_)) >= 0


  ##
  # @if jp
  #
  # @brief id  ConnectorProfile õ
  #
  # Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
  #  ConnectorProfile Υꥹ椫õ
  # ⤷Ʊ id  ConnectorProfile ʤС ConnectorProfile
  # ֤롣
  #
  # @param self
  # @param id_  connector_id
  #
  # @return connector_id  ConnectorProfile
  #
  # @else
  #
  # @brief Find ConnectorProfile with id
  #
  # This operation returns ConnectorProfile with the given id from Port's
  # ConnectorProfiles' list.
  # If the ConnectorProfile with connector id that is identical with the
  # given id does not exist, empty ConnectorProfile is returned.
  #
  # @param id the connector_id to be searched in Port's ConnectorProfiles
  #
  # @return CoonectorProfile with connector_id
  #
  # @endif
  # ConnectorProfile findConnProfile(const char* id);
  def findConnProfile(self, id_):
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                            self.find_conn_id(id_))
    if index < 0 or index >= len(self._profile.connector_profiles):
      return RTC.ConnectorProfile("","",[],[])

    return self._profile.connector_profiles[index]


  ##
  # @if jp
  #
  # @brief id  ConnectorProfile õ
  #
  # Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
  #  ConnectorProfile Υꥹ椫õǥå֤
  # ⤷Ʊ id  ConnectorProfile ʤС-1 ֤
  #
  # @param self
  # @param id_  connector_id
  #
  # @return Port  ConnectorProfile ꥹȤΥǥå
  #
  # @else
  #
  # @brief Find ConnectorProfile with id
  #
  # This operation returns ConnectorProfile with the given id from Port's
  # ConnectorProfiles' list.
  # If the ConnectorProfile with connector id that is identical with the
  # given id does not exist, empty ConnectorProfile is returned.
  #
  # @param id the connector_id to be searched in Port's ConnectorProfiles
  #
  # @return The index of ConnectorProfile of the Port
  #
  # @endif
  # CORBA::Long findConnProfileIndex(const char* id);
  def findConnProfileIndex(self, id_):
    return OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                           self.find_conn_id(id_))


  ##
  # @if jp
  #
  # @brief ConnectorProfile ɲä⤷Ϲ
  #
  # Υڥ졼Ϳ줿Ϳ줿 ConnectorProfile 
  # Port ɲä⤷Ϲ¸롣
  # Ϳ줿 ConnectorProfile  connector_id Ʊ ID 
  # ConnectorProfile ꥹȤˤʤСꥹȤɲä
  # Ʊ ID ¸ߤ ConnectorProfile ¸롣
  #
  # @param self
  # @param connector_profile ɲä⤷Ϲ ConnectorProfile
  #
  # @else
  #
  # @brief Append or update the ConnectorProfile list
  #
  # This operation appends or updates ConnectorProfile of the Port
  # by the given ConnectorProfile.
  # If the connector_id of the given ConnectorProfile does not exist
  # in the Port's ConnectorProfile list, the given ConnectorProfile would be
  # append to the list. If the same id exists, the list would be updated.
  #
  # @param connector_profile the ConnectorProfile to be appended or updated
  #
  # @endif
  # void updateConnectorProfile(const ConnectorProfile& connector_profile);
  def updateConnectorProfile(self, connector_profile):
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                            self.find_conn_id(connector_profile.connector_id))

    if index < 0:
      OpenRTM_aist.CORBA_SeqUtil.push_back(self._profile.connector_profiles,
                                           connector_profile)
    else:
      self._profile.connector_profiles[index] = connector_profile


  ##
  # @if jp
  #
  # @brief ConnectorProfile 
  #
  # Υڥ졼 Port  PortProfile ݻƤ
  # ConnectorProfileList ΤͿ줿 id  ConnectorProfile
  # 롣
  #
  # @param self
  # @param id_  ConnectorProfile  id
  #
  # @return ˺Ǥ true
  #         ꤷ ConnectorProfile Ĥʤ false ֤
  #
  # @else
  #
  # @brief Delete the ConnectorProfile
  #
  # This operation deletes a ConnectorProfile specified by id from
  # ConnectorProfileList owned by PortProfile of this Port.
  #
  # @param id The id of the ConnectorProfile to be deleted.
  #
  # @endif
  # bool eraseConnectorProfile(const char* id);
  def eraseConnectorProfile(self, id_):
    guard = OpenRTM_aist.ScopedLock(self._profile_mutex)

    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.connector_profiles,
                                            self.find_conn_id(id_))

    if index < 0:
      return False

    OpenRTM_aist.CORBA_SeqUtil.erase(self._profile.connector_profiles, index)

    return True


  ##
  # @if jp
  #
  # @brief PortInterfaceProfile  󥿡եϿ
  #
  # Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
  # ˥󥿡եξɲä롣
  # ξϡget_port_profile() ä PortProfile Τ
  # PortInterfaceProfile ͤѹΤߤǤꡢºݤ˥󥿡ե
  # 󶡤׵ᤷꤹˤϡ֥饹ǡ publishInterface() ,
  #  subscribeInterface() δؿŬڤ˥С饤ɤ󥿡ե
  # 󶡡׵ԤʤФʤʤ
  #
  # 󥿡ե(Υ󥹥)̾ Port ǰդǤʤФʤʤ
  # Ʊ̾Υ󥿡եǤϿƤ硢δؿ false 
  # ֤
  #
  # @param self
  # @param instance_name 󥿡եΥ󥹥󥹤̾
  # @param type_name 󥿡եη̾
  # @param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
  #
  # @return 󥿡եϿ̡
  #         Ʊ̾Υ󥿡եϿƤ false ֤
  #
  # @else
  #
  # @brief Append an interface to the PortInterfaceProfile
  #
  # This operation appends interface information to the PortInterfaceProfile
  # that is owned by the Port.
  # The given interfaces information only updates PortInterfaceProfile of
  # PortProfile that is obtained through get_port_profile().
  # In order to provide and require interfaces, proper functions (for
  # example publishInterface(), subscribeInterface() and so on) should be
  # overridden in subclasses, and these functions provide concrete interface
  # connection and disconnection functionality.
  #
  # The interface (instance) name have to be unique in the Port.
  # If the given interface name is identical with stored interface name,
  # this function returns false.
  #
  # @param name The instance name of the interface.
  # @param type_name The type name of the interface.
  # @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
  #
  # @return false would be returned if the same name is already registered.
  #
  # @endif
  # bool appendInterface(const char* name, const char* type_name,
  #			 PortInterfacePolarity pol);
  def appendInterface(self, instance_name, type_name, pol):
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.interfaces,
                                            self.find_interface(instance_name, pol))

    if index >= 0:
      return False

    # setup PortInterfaceProfile
    prof = RTC.PortInterfaceProfile(instance_name, type_name, pol)
    OpenRTM_aist.CORBA_SeqUtil.push_back(self._profile.interfaces, prof)

    return True


  ##
  # @if jp
  #
  # @brief PortInterfaceProfile 饤󥿡եϿ
  #
  # Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
  # 饤󥿡եξ롣
  #
  # @param self
  # @param name 󥿡եΥ󥹥󥹤̾
  # @param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
  #
  # @return 󥿡ե̡
  #         󥿡եϿƤʤ false ֤
  #
  # @else
  #
  # @brief Delete an interface from the PortInterfaceProfile
  #
  # This operation deletes interface information from the
  # PortInterfaceProfile that is owned by the Port.
  #
  # @param name The instance name of the interface.
  # @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
  #
  # @return false would be returned if the given name is not registered.
  #
  # @endif
  # bool deleteInterface(const char* name, PortInterfacePolarity pol);
  def deleteInterface(self, name, pol):
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._profile.interfaces,
                                            self.find_interface(name, pol))

    if index < 0:
      return False

    OpenRTM_aist.CORBA_SeqUtil.erase(self._profile.interfaces, index)
    return True


  ##
  # @if jp
  #
  # @brief PortProfile  properties  NameValue ͤɲä
  #
  # PortProfile  properties  NameValue ͤɲä롣
  # ɲäǡηValueTypeǻꤹ롣
  #
  # @param self
  # @param key properties  name
  # @param value properties  value
  #
  # @else
  #
  # @brief Add NameValue data to PortProfile's properties
  #
  # @param key The name of properties
  # @param value The value of properties
  #
  # @endif
  #  template <class ValueType>
  #  void addProperty(const char* key, ValueType value)
  def addProperty(self, key, value):
    OpenRTM_aist.CORBA_SeqUtil.push_back(self._profile.properties,
                                         OpenRTM_aist.NVUtil.newNV(key, value))

  # void appendProperty(const char* key, const char* value)
  def appendProperty(self, key, value):
    OpenRTM_aist.NVUtil.appendStringValue(self._profile.properties, key, value)


  #============================================================
  # Functor
  #============================================================

  ##
  # @if jp
  # @class if_name
  # @brief instance_name  PortInterfaceProfile õ Functor
  # @else
  # @brief A functor to find a PortInterfaceProfile named instance_name
  # @endif
  class if_name:
    def __init__(self, name):
      self._name = name

    def __call__(self, prof):
      return str(self._name) == str(prof.instance_name)
    

  ##
  # @if jp
  # @class find_conn_id
  # @brief id  ConnectorProfile õ Functor
  # @else
  # @brief A functor to find a ConnectorProfile named id
  # @endif
  class find_conn_id:
    def __init__(self, id_):
      """
       \param id_(string)
      """
      self._id = id_

    def __call__(self, cprof):
      """
       \param cprof(RTC.ConnectorProfile)
      """
      return str(self._id) == str(cprof.connector_id)

  ##
  # @if jp
  # @class find_port_ref
  # @brief 󥹥ȥ饯 port_ref Ʊ֥ȻȤõ Functor
  # @else
  # @brief A functor to find the object reference that is identical port_ref
  # @endif
  class find_port_ref:
    def __init__(self, port_ref):
      """
       \param port_ref(RTC.PortService)
      """
      self._port_ref = port_ref

    def __call__(self, port_ref):
      """
       \param port_ref(RTC.PortService)
      """
      return self._port_ref._is_equivalent(port_ref)

  ##
  # @if jp
  # @class connect_func
  # @brief Port ³Ԥ Functor
  # @else
  # @brief A functor to connect Ports
  # @endif
  class connect_func:
    def __init__(self, p, prof):
      """
       \param p(RTC.PortService)
       \param prof(RTC.ConnectorProfile)
      """
      self._port_ref = p
      self._connector_profile = prof
      self.return_code = RTC.RTC_OK

    def __call__(self, p):
      """
       \param p(RTC.PortService)
      """
      if not self._port_ref._is_equivalent(p):
        retval = p.notify_connect(self._connector_profile)
        if retval != RTC.RTC_OK:
          self.return_code = retval

  ##
  # @if jp
  # @class disconnect_func
  # @brief Port ³Ԥ Functor
  # @else
  # @brief A functor to disconnect Ports
  # @endif
  class disconnect_func:
    def __init__(self, p, prof):
      """
       \param p(RTC.PortService)
       \param prof(RTC.ConnectorProfile)
      """
      self._port_ref = p
      self._connector_profile = prof
      self.return_code = RTC.RTC_OK
      
    def __call__(self, p):
      """
       \param p(RTC.PortService)
      """
      if not self._port_ref._is_equivalent(p):
        retval = p.disconnect(self._connector_profile.connector_id)
        if retval != RTC.RTC_OK:
          self.return_code = retval

  ##
  # @if jp
  # @class disconnect_all_func
  # @brief Port ³Ԥ Functor
  # @else
  # @brief A functor to disconnect all Ports
  # @endif
  class disconnect_all_func:
    def __init__(self, p):
      """
       \param p(OpenRTM_aist.PortBase)
      """
      self.return_code = RTC.RTC_OK
      self._port = p

    def __call__(self, p):
      """
       \param p(RTC.ConnectorProfile)
      """
      retval = self._port.disconnect(p.connector_id)
      if retval != RTC.RTC_OK:
        self.return_code = retval

  ##
  # @if jp
  # @class find_interface
  # @brief name  polarity  interface õ Functor
  # @else
  # @brief A functor to find interface from name and polarity
  # @endif
  class find_interface:
    def __init__(self, name, pol):
      """
       \param name(string)
       \param pol(RTC.PortInterfacePolarity)
      """
      self._name = name
      self._pol = pol

    def __call__(self, prof):
      """
       \param prof(RTC.PortInterfaceProfile)
      """
      name = prof.instance_name
      return (str(self._name) == str(name)) and (self._pol == prof.polarity)
