#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file RTObject.py
# @brief RT component base class
# @date $Date: $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2006-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.



from omniORB import any
from omniORB import CORBA
import string
import sys

import OpenRTM, OpenRTM__POA
import RTC,RTC__POA
import SDOPackage,SDOPackage__POA
import OpenRTM_aist

ECOTHER_OFFSET = 1000

default_conf = [
  "implementation_id","",
  "type_name",         "",
  "description",       "",
  "version",           "",
  "vendor",            "",
  "category",          "",
  "activity_type",     "",
  "max_instance",      "",
  "language",          "",
  "lang_type",         "",
  "conf",              "",
  "" ]



##
# @if jp
# @brief RTݡͥȥ饹
#
# RTݡͥȤΥ١Ȥʤ륯饹
# Robotic Technology Component  lightweightRTComponentμ饹
# ݡͥȤεǽ󶡤 ComponentAction 󥿡ե
# ݡͥȤΥ饤եԤ LightweightRTObject μ
# 󶡤롣
# ºݤ˥桼ݡͥȤˤϡExecution Semantics б
# ƥ֥饹Ѥ롣<BR>
# (μǤ Periodic Sampled Data Processing ΤߥݡȤƤ뤿ᡢ
#  dataFlowComponent ľܷѾƤ)
#
# @since 0.2.0
#
# @else
#
# @endif
class RTObject_impl(OpenRTM__POA.DataFlowComponent):



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param manager ޥ͡㥪֥(ǥե:None)
  # @param orb ORB(ǥե:None)
  # @param poa POA(ǥե:None)
  #
  # @else
  #
  # @brief Consructor
  #
  # @param orb ORB
  # @param poa POA
  #
  # @endif
  def __init__(self, manager=None, orb=None, poa=None):
    if manager:
      self._manager = manager
      self._orb = self._manager.getORB()
      self._poa = self._manager.getPOA()
      self._portAdmin = OpenRTM_aist.PortAdmin(self._manager.getORB(),self._manager.getPOA())
    else:
      self._manager = None
      self._orb = orb
      self._poa = poa
      self._portAdmin = OpenRTM_aist.PortAdmin(self._orb,self._poa)
      
    if self._manager:
      self._rtcout = self._manager.getLogbuf("rtobject")
    else:
      self._rtcout = OpenRTM_aist.Manager.instance().getLogbuf("rtobject")

    self._created = True
    self._properties = OpenRTM_aist.Properties(defaults_str=default_conf)
    self._configsets = OpenRTM_aist.ConfigAdmin(self._properties.getNode("conf"))
    self._profile = RTC.ComponentProfile("","","","","","",[],None,[])
    
    self._SdoConfigImpl = OpenRTM_aist.Configuration_impl(self._configsets)
    self._SdoConfig = self._SdoConfigImpl.getObjRef()
    self._execContexts = []
    self._objref = self._this()
    self._sdoOwnedOrganizations = [] #SDOPackage.OrganizationList()
    self._sdoSvcProfiles        = [] #SDOPackage.ServiceProfileList()
    self._sdoOrganizations      = [] #SDOPackage.OrganizationList()
    self._sdoStatus             = [] #SDOPackage.NVList()
    self._ecMine  = []
    self._ecOther = []
    self._eclist  = []
    return


  ##
  # @if jp
  #
  # @brief ǥȥ饯
  #
  # @param self
  # 
  # @else
  # 
  # @brief destructor
  # 
  # @endif
  def __del__(self):
    return


  #============================================================
  # Overridden functions
  #============================================================

  ##
  # @if jp
  #
  # @brief ѥХåؿ
  # 
  # ComponentAction::on_initialize ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤνϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  #
  # @param self
  # 
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onInitialize(self):
    self._rtcout.RTC_TRACE("onInitialize()")
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief λѥХåؿ
  # 
  # ComponentAction::on_finalize ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤνλϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  #
  # @param self
  # 
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onFinalize(self):
    self._rtcout.RTC_TRACE("onFinalize()")
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ϽѥХåؿ
  # 
  # ComponentAction::on_startup ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤγϽϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onStartup(self, ec_id):
    self._rtcout.RTC_TRACE("onStartup(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ߽ѥХåؿ
  # 
  # ComponentAction::on_shutdown ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤ߽ϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onShutdown(self, ec_id):
    self._rtcout.RTC_TRACE("onShutdown(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ѥХåؿ
  # 
  # ComponentAction::on_activated ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤγϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onActivated(self, ec_id):
    self._rtcout.RTC_TRACE("onActivated(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ѥХåؿ
  # 
  # ComponentAction::on_deactivated ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onDeactivated(self, ec_id):
    self._rtcout.RTC_TRACE("onDeactivated(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ѥХåؿ
  # 
  # DataFlowComponentAction::on_execute ƤФ줿ݤ˼¹Ԥ
  # Хåؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤμϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣<BR>
  # ܴؿ Periodic Sampled Data Processing ˤ Two-Pass Execution
  # ܤμ¹ԥѥȤŪ˸ƤӽФ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onExecute(self, ec_id):
    self._rtcout.RTC_TRACE("onExecute(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ǽѥХåؿ
  # 
  # ComponentAction::on_aborting ƤФ줿ݤ˼¹Ԥ륳Хå
  # ؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤǽϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onAborting(self, ec_id):
    self._rtcout.RTC_TRACE("onAborting(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief 顼ѥХåؿ
  # 
  # ComponentAction::on_error ƤФ줿ݤ˼¹Ԥ륳Хåؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤΥ顼ϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onError(self, ec_id):
    self._rtcout.RTC_TRACE("onError(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ꥻåȽѥХåؿ
  # 
  # ComponentAction::on_reset ƤФ줿ݤ˼¹Ԥ륳Хåؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤΥꥻåȽϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣
  # 
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onReset(self, ec_id):
    self._rtcout.RTC_TRACE("onReset(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ѹѥХåؿ
  # 
  # DataFlowComponentAction::on_state_update ƤФ줿ݤ˼¹Ԥ
  # Хåؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤξѹϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣<BR>
  # ܴؿ Periodic Sampled Data Processing ˤ Two-Pass Execution
  # ܤμ¹ԥѥȤŪ˸ƤӽФ롣
  #
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  # 
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onStateUpdate(self, ec_id):
    self._rtcout.RTC_TRACE("onStateupdate(%s)",ec_id)
    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief ưѹѥХåؿ
  # 
  # DataFlowComponentAction::on_rate_changed ƤФ줿ݤ˼¹Ԥ
  # Хåؿ<BR>
  # ܴؿ̵ RTC::RTC_OK ֤褦˥ߡƤΤǡ
  # ƥݡͥȤμºݤξѹϡܴؿ򥪡С饤ɤƼ
  # ɬפ롣<BR>
  # ܴؿ Periodic Sampled Data Processing ˤ ExecutionContext 
  # ¹Ԥ줿ݤ˸ƤӽФ롣
  #
  # @param self
  # @param ec_id äƤ ExecutionContext  ID
  # 
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  # 
  # @endif
  def onRateChanged(self, ec_id):
    self._rtcout.RTC_TRACE("onRatechanged(%s)",ec_id)
    return RTC.RTC_OK 


  #============================================================
  # RTC::LightweightRTObject
  #============================================================

  ##
  # @if jp
  #
  # @brief [CORBA interface] RTC
  #
  # Υڥ졼ƤӽФη̤ȤơComponentAction::on_initialize
  # ХåؿƤФ롣
  # 
  # 
  # - RTC  Created֤ξ߽Ԥ롣¾ξ֤ˤˤ
  #   ReturnCode_t::PRECONDITION_NOT_MET ֤ƤӽФϼԤ롣
  # - Υڥ졼 RTC Υߥɥ륦ƤФ뤳ȤꤷƤꡢ
  #   ץꥱȯԤľܤΥڥ졼Ƥ֤Ȥ
  #   Ƥʤ
  #
  # @param self
  # 
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  #
  # @brief Initialize the RTC that realizes this interface.
  #
  # The invocation of this operation shall result in the invocation of the
  # callback ComponentAction::on_initialize.
  #
  # Constraints
  # - An RTC may be initialized only while it is in the Created state. Any
  #   attempt to invoke this operation while in another state shall fail
  #   with ReturnCode_t::PRECONDITION_NOT_MET.
  # - Application developers are not expected to call this operation
  #   directly; it exists for use by the RTC infrastructure.
  #
  # @return
  # 
  # @endif
  def initialize(self):
    self._rtcout.RTC_TRACE("initialize()")

    ec_args = self._properties.getProperty("exec_cxt.periodic.type")
    ec_args += "?"
    ec_args += "rate="
    ec_args += self._properties.getProperty("exec_cxt.periodic.rate")

    ec = OpenRTM_aist.Manager.instance().createContext(ec_args)
    if ec is None:
      return RTC.RTC_ERROR

    ec.set_rate(float(self._properties.getProperty("exec_cxt.periodic.rate")))
    self._eclist.append(ec)
    ecv = ec.getObjRef()
    if CORBA.is_nil(ecv):
      return RTC.RTC_ERROR

    ec.bindComponent(self)

    # at least one EC must be attached
    if len(self._ecMine) == 0:
      return RTC.PRECONDITION_NOT_MET

    ret = self.on_initialize()
    if ret is not RTC.RTC_OK:
      return ret
    self._created = False

    # -- entering alive state --
    for i in range(len(self._ecMine)):
      self._rtcout.RTC_DEBUG("EC[%d] starting.", i)
      self._ecMine[i].start()

    # ret must be RTC_OK
    return ret


  ##
  # @if jp
  #
  # @brief [CORBA interface] RTC λ
  #
  # Υڥ졼ƤӽФη̤Ȥ ComponentAction::on_finalize()
  # ƤӽФ
  #
  # 
  # - RTC  ExecutionContext ˽°Ƥ֤Ͻλʤξϡ
  #   ޤǽ ExecutionContextOperations::remove_component ˤäƻä
  #   ʤФʤʤʳξϡΥڥ졼ƤӽФ
  #   ʤ ReturnCode_t::PRECONDITION_NOT_ME ǼԤ롣
  # - RTC  Created ֤Ǥ硢λϹԤʤ
  #   ξ硢Υڥ졼ƤӽФϤʤ
  #   ReturnCode_t::PRECONDITION_NOT_MET ǼԤ롣
  # - Υڥ졼RTCΥߥɥ륦ƤФ뤳ȤꤷƤꡢ
  #   ץꥱȯԤľܤΥڥ졼Ƥ֤Ȥ
  #   Ƥʤ
  #
  # @param self
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  #
  # @brief Finalize the RTC for preparing it for destruction
  # 
  # This invocation of this operation shall result in the invocation of the
  # callback ComponentAction::on_finalize.
  #
  # Constraints
  # - An RTC may not be finalized while it is participating in any execution
  #   context. It must first be removed with 
  #   ExecutionContextOperations::remove_component. Otherwise, this operation
  #   shall fail with ReturnCode_t::PRECONDITION_NOT_MET. 
  # - An RTC may not be finalized while it is in the Created state. Any 
  #   attempt to invoke this operation while in that state shall fail with 
  #   ReturnCode_t::PRECONDITION_NOT_MET.
  # - Application developers are not expected to call this operation directly;
  #  it exists for use by the RTC infrastructure.
  #
  # @return
  # 
  # @endif
  def finalize(self):
    self._rtcout.RTC_TRACE("finalize()")
    if self._created:
      return RTC.PRECONDITION_NOT_MET
    
    # Return RTC::PRECONDITION_NOT_MET,
    # When the component is registered in ExecutionContext.
    if len(self._ecOther) != 0:
      return RTC.PRECONDITION_NOT_MET

    ret = self.on_finalize()
    self.shutdown()
    return ret


  ##
  # @if jp
  #
  # @brief [CORBA interface] RTC ʡǤ ExecutionContext 
  #        ߤΥƥĤȶ˽λ
  #
  #  RTC ʡǤ뤹٤Ƥμ¹ԥƥȤߤ롣
  #  RTC ¾μ¹ԥƥȤͭ RTC °¹ԥƥ
  # (i.e. ¹ԥƥȤͭ RTC Ϥʤμ¹ԥƥȤ
  # ʡǤ롣)˻äƤ硢 RTC ϤΥƥȾ
  # ʤФʤʤ
  # RTC ¹Τɤ ExecutionContext Ǥ Active ֤ǤϤʤʤä塢
  #  RTC Ȥ˴ޤޤ RTC λ롣
  # 
  # 
  # - RTC ƤʤСλ뤳ȤϤǤʤ
  #   Created ֤ˤ RTC  exit() ƤӽФ硢
  #   ReturnCode_t::PRECONDITION_NOT_MET ǼԤ롣
  #
  # @param self
  #
  # @return ReturnCode_t Υ꥿󥳡
  # 
  # @else
  #
  # @brief Stop the RTC's execution context(s) and finalize it along with its
  #        contents.
  # 
  # Any execution contexts for which the RTC is the owner shall be stopped. 
  # If the RTC participates in any execution contexts belonging to another
  # RTC that contains it, directly or indirectly (i.e. the containing RTC
  # is the owner of the ExecutionContext), it shall be deactivated in those
  # contexts.
  # After the RTC is no longer Active in any Running execution context, it
  # and any RTCs contained transitively within it shall be finalized.
  #
  # Constraints
  # - An RTC cannot be exited if it has not yet been initialized. Any
  #   attempt to exit an RTC that is in the Created state shall fail with
  #   ReturnCode_t::PRECONDITION_NOT_MET.
  #
  # @return
  # 
  # @endif
  def exit(self):
    self._rtcout.RTC_TRACE("exit()")
    if self._created:
      return RTC.PRECONDITION_NOT_MET

    # deactivate myself on owned EC
    OpenRTM_aist.CORBA_SeqUtil.for_each(self._ecMine,
                                        self.deactivate_comps(self._objref))
    # deactivate myself on other EC
    OpenRTM_aist.CORBA_SeqUtil.for_each(self._ecOther,
                                        self.deactivate_comps(self._objref))

    # stop and detach myself from owned EC
    for ec in self._ecMine:
      ec.stop()
    #  ec.remove_component(self._this())

    # detach myself from other EC
    for ec in self._ecOther:
      # ec.stop()
      ec.remove_component(self._this())

    return self.finalize()


  ##
  # @if jp
  #
  # @brief [CORBA interface] RTC  Alive ֤Ǥ뤫ɤǧ롣
  #
  # RTC ꤷ ExecutionContext Ф Alive֤Ǥ뤫ɤǧ롣
  # RTC ξ֤ Active Ǥ뤫Inactive Ǥ뤫Error Ǥ뤫ϼ¹
  # ExecutionContext ˰¸롣ʤ ExecutionContext ФƤ
  # Active  ֤ǤäƤ⡢¾ ExecutionContext ФƤ Inactive ֤
  # ʤ⤢ꤨ롣äơΥڥ졼ϻꤵ줿
  # ExecutionContext 䤤碌ơ RTC ξ֤ ActiveInactive
  # Error ξˤ Alive ֤Ȥ֤
  #
  # @param self
  #
  # @param exec_context о ExecutionContext ϥɥ
  #
  # @return Alive ֳǧ
  #
  # @else
  #
  # @brief Confirm whether RTC is an Alive state or NOT.
  #
  # A component is alive or not regardless of the execution context from
  # which it is observed. However, whether or not it is Active, Inactive,
  # or in Error is dependent on the execution context(s) in which it is
  # running. That is, it may be Active in one context but Inactive in
  # another. Therefore, this operation shall report whether this RTC is
  # either Active, Inactive or in Error; which of those states a component
  # is in with respect to a particular context may be queried from the
  # context itself.
  #
  # @return Result of Alive state confirmation
  #
  # @endif
  # virtual CORBA::Boolean is_alive(ExecutionContext_ptr exec_context)
  def is_alive(self, exec_context):
    self._rtcout.RTC_TRACE("is_alive()")
    for ec in self._ecMine:
      if exec_context._is_equivalent(ec):
        return True

    for ec in self._ecOther:
      if exec_context._is_equivalent(ec):
        return True

    return False


  ##
  # @if jp
  # @brief [CORBA interface] ExecutionContextList
  #
  #  RTC ͭ ExecutionContext ΥꥹȤ롣
  #
  # @param self
  #
  # @return ExecutionContext ꥹ
  #
  # @else
  # @brief [CORBA interface] Get ExecutionContextList.
  #
  # This operation returns a list of all execution contexts owned by this RTC.
  #
  # @return ExecutionContext List
  #
  # @endif
  #def get_contexts(self):
  #  execlist = []
  #  OpenRTM_aist.CORBA_SeqUtil.for_each(self._execContexts, self.ec_copy(execlist))
  #  return execlist


  ##
  # @if jp
  # @brief [CORBA interface] ExecutionContext
  #
  # ꤷϥɥ ExecutionContext 롣
  # ϥɥ뤫 ExecutionContext ؤΥޥåԥ󥰤ϡ RTC 󥹥󥹤
  # ͭǤ롣ϥɥϤ RTC  attach_context ݤ˼Ǥ롣
  #
  # @param self
  # @param ec_id о ExecutionContext ϥɥ
  #
  # @return ExecutionContext
  #
  # @else
  # @brief [CORBA interface] Get ExecutionContext.
  #
  # Obtain a reference to the execution context represented by the given 
  # handle.
  # The mapping from handle to context is specific to a particular RTC 
  # instance. The given handle must have been obtained by a previous call to 
  # attach_context on this RTC.
  #
  # @param ec_id ExecutionContext handle
  #
  # @return ExecutionContext
  #
  # @endif
  # virtual ExecutionContext_ptr get_context(UniqueId exec_handle)
  def get_context(self, ec_id):
    global ECOTHER_OFFSET

    self._rtcout.RTC_TRACE("get_context(%s)", ec_id)
    # owned EC
    if ec_id < ECOTHER_OFFSET:
      if ec_id < len(self._ecMine):
        return self._ecMine[ec_id]
      else:
        return RTC.ExecutionContext._nil

    # participating EC
    index = ec_id - ECOTHER_OFFSET

    if index < len(self._ecOther):
      return self._ecOther[index]

    return RTC.ExecutionContext._nil


  ##
  # @if jp
  # @brief [CORBA interface] ͭ ExecutionContextList 
  #
  #  RTC ͭ ExecutionContext ΥꥹȤ롣
  #
  # @return ExecutionContext ꥹ
  #
  # @else
  # @brief [CORBA interface] Get ExecutionContextList.
  #
  # This operation returns a list of all execution contexts owned by this
  # RTC.
  #
  # @return ExecutionContext List
  #
  # @endif
  # virtual ExecutionContextList* get_owned_contexts()
  def get_owned_contexts(self):
    self._rtcout.RTC_TRACE("get_owned_contexts()")
    execlist = []
    OpenRTM_aist.CORBA_SeqUtil.for_each(self._ecMine, self.ec_copy(execlist))
    return execlist # ExecutionContextList* 


  ##
  # @if jp
  # @brief [CORBA interface] äƤ ExecutionContextList 
  #
  #  RTC äƤ ExecutionContext ΥꥹȤ롣
  #
  # @return ExecutionContext ꥹ
  #
  # @else
  # @brief [CORBA interface] Get participating ExecutionContextList.
  #
  # This operation returns a list of all execution contexts in
  # which this RTC participates.
  #
  # @return ExecutionContext List
  #
  # @endif
  # virtual ExecutionContextList* get_participating_contexts()
  def get_participating_contexts(self):
    self._rtcout.RTC_TRACE("get_participating_contexts()")
    execlist = []
    OpenRTM_aist.CORBA_SeqUtil.for_each(self._ecOther, self.ec_copy(execlist))
    
    return execlist # ExecutionContextList*


  #
  # @if jp
  # @brief [CORBA interface] ExecutionContext Υϥɥ֤
  #
  # @param ExecutionContext ¹ԥƥ
  #
  # @return ExecutionContextHandle
  #
  # Ϳ줿¹ԥƥȤ˴Ϣդ줿ϥɥ֤
  #
  # @else
  # @brief [CORBA interface] Return a handle of a ExecutionContext
  #
  # @param ExecutionContext
  #
  # @return ExecutionContextHandle
  #
  # This operation returns a handle that is associated with the given
  # execution context.
  #
  # @endif
  #
  # virtual ExecutionContextHandle_t
  #   get_context_handle(ExecutionContext_ptr cxt)
  def get_context_handle(self, cxt):
    self._rtcout.RTC_TRACE("get_context_handle()")
    # ec_id 0 : owned context
    # ec_id 1-: participating context
    if cxt._is_equivalent(self._ecMine[0]):
      return 0

    num = OpenRTM_aist.CORBA_SeqUtil.find(self._ecOther, self.ec_find(cxt))
    return num + 1 #ExecutionContextHandle_t


  #============================================================
  # RTC::RTObject
  #============================================================

  ##
  # @if jp
  #
  # @brief [RTObject CORBA interface] ݡͥȥץե
  #
  # ݡͥȤΥץե֤ 
  #
  # @param self
  #
  # @return ݡͥȥץե
  #
  # @else
  #
  # @brief [RTObject CORBA interface] Get RTC's profile
  #
  # This operation returns the ComponentProfile of the RTC
  #
  # @return ComponentProfile
  #
  # @endif
  # virtual ComponentProfile* get_component_profile()
  def get_component_profile(self):
    self._rtcout.RTC_TRACE("get_component_profile()")
    try:
      return RTC.ComponentProfile(self._properties.getProperty("instance_name"),
                                  self._properties.getProperty("type_name"),
                                  self._properties.getProperty("description"),
                                  self._properties.getProperty("version"),
                                  self._properties.getProperty("vendor"),
                                  self._properties.getProperty("category"),
                                  self._portAdmin.getPortProfileList(),
                                  self._profile.parent,
                                  self._profile.properties)
      # return RTC.ComponentProfile(self._profile.instance_name,
      #               self._profile.type_name,
      #               self._profile.description,
      #               self._profile.version,
      #               self._profile.vendor,
      #               self._profile.category,
      #               self._portAdmin.getPortProfileList(),
      #               self._profile.parent,
      #               self._profile.properties)
    
    except:
      #traceback.print_exception(*sys.exc_info())
      self._rtcout.RTC_ERROR(sys.exc_info()[0])

    assert(False)
    return None


  ##
  # @if jp
  #
  # @brief [RTObject CORBA interface] ݡȤ
  #
  # ݡͥȤͭݡȤλȤ֤
  #
  # @param self
  #
  # @return ݡȥꥹ
  #
  # @else
  #
  # @brief [RTObject CORBA interface] Get Ports
  #
  # This operation returns a list of the RTCs ports.
  #
  # @return PortList
  #
  # @endif
  # virtual PortServiceList* get_ports()
  def get_ports(self):
    self._rtcout.RTC_TRACE("get_ports()")
    try:
      return self._portAdmin.getPortServiceList()
    except:
      #traceback.print_exception(*sys.exc_info())
      self._rtcout.RTC_ERROR(sys.exc_info()[0])

    assert(False)
    return []



  # RTC::ComponentAction

  ##
  # @if jp
  # @brief [CORBA interface] ExecutionContextattach
  #
  # ꤷ ExecutionContext ˤ RTC °롣 RTC ȴϢ 
  # ExecutionContext Υϥɥ֤
  # Υڥ졼ϡExecutionContextOperations::add_component ƤФ줿
  # ݤ˸ƤӽФ롣֤줿ϥɥ¾Υ饤ȤǻѤ뤳Ȥ
  # Ƥʤ
  #
  # @param self
  # @param exec_context ° ExecutionContext
  #
  # @return ExecutionContext ϥɥ
  #
  # @else
  # @brief [CORBA interface] Attach ExecutionContext.
  #
  # Inform this RTC that it is participating in the given execution context. 
  # Return a handle that represents the association of this RTC with the 
  # context.
  # This operation is intended to be invoked by 
  # ExecutionContextOperations::add_component. It is not intended for use by 
  # other clients.
  #
  # @param exec_context Prticipating ExecutionContext
  #
  # @return ExecutionContext Handle
  #
  # @endif
  # UniqueId attach_context(ExecutionContext_ptr exec_context)
  def attach_context(self, exec_context):
    self._rtcout.RTC_TRACE("attach_context()")
    # ID: 0 - (offset-1) : owned ec
    # ID: offset -       : participating ec
    # owned       ec index = ID
    # participate ec index = ID - offset
    ecs = exec_context._narrow(RTC.ExecutionContextService)
    if CORBA.is_nil(ecs):
      return -1
    
    # if m_ecOther has nil element, insert attached ec to there.
    for i in range(len(self._ecOther)):
      if CORBA.is_nil(self._ecOther[i]):
        self._ecOther[i] = ecs
        return i + ECOTHER_OFFSET

    # no space in the list, push back ec to the last.
    OpenRTM_aist.CORBA_SeqUtil.push_back(self._ecOther,ecs)
    
    return long(len(self._ecOther) - 1 + ECOTHER_OFFSET)


  # UniqueId bindContext(ExecutionContext_ptr exec_context);
  def bindContext(self, exec_context):
    self._rtcout.RTC_TRACE("bindContext()")
    # ID: 0 - (offset-1) : owned ec
    # ID: offset -       : participating ec
    # owned       ec index = ID
    # participate ec index = ID - offset
    ecs = exec_context._narrow(RTC.ExecutionContextService)

    if CORBA.is_nil(ecs):
      return -1
    
    # if m_ecMine has nil element, insert attached ec to there.
    for i in range(len(self._ecMine)):
      if CORBA.is_nil(self._ecMine[i]):
        self._ecMine[i] = ecs
        return i
        #return i + ECOTHER_OFFSET

    # no space in the list, push back ec to the last.
    OpenRTM_aist.CORBA_SeqUtil.push_back(self._ecMine,ecs)
    
    return long(len(self._ecMine) - 1)
    #return long(len(self._ecMine) - 1 + ECOTHER_OFFSET)


  ##
  # @if jp
  # @brief [CORBA interface] ExecutionContextdetach
  #
  # ꤷ ExecutionContext 餳 RTC ν°롣
  # Υڥ졼ϡExecutionContextOperations::remove_component Ƥ
  # 줿ݤ˸ƤӽФ롣֤줿ϥɥ¾Υ饤ȤǻѤ뤳Ȥ
  # ꤷƤʤ
  # 
  # 
  # - ꤵ줿 ExecutionContext  RTC Ǥ˽°Ƥʤˤϡ
  #   ReturnCode_t::PRECONDITION_NOT_MET ֤롣
  # - ꤵ줿 ExecutionContext ˤФ RTC Active ֤Ǥ
  #   ˤϡ ReturnCode_t::PRECONDITION_NOT_MET ֤롣
  #
  # @param self
  # @param ec_id о ExecutionContextϥɥ
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  # @brief [CORBA interface] Attach ExecutionContext.
  #
  # Inform this RTC that it is no longer participating in the given execution 
  # context.
  # This operation is intended to be invoked by 
  # ExecutionContextOperations::remove_component. It is not intended for use 
  # by other clients.
  # Constraints
  # - This operation may not be invoked if this RTC is not already 
  #   participating in the execution context. Such a call shall fail with 
  #   ReturnCode_t::PRECONDITION_NOT_MET.
  # - This operation may not be invoked if this RTC is Active in the indicated
  #   execution context. Otherwise, it shall fail with 
  #   ReturnCode_t::PRECONDITION_NOT_MET.
  #
  # @param ec_id Dettaching ExecutionContext Handle
  #
  # @return
  #
  # @endif
  # ReturnCode_t detach_context(UniqueId exec_handle)
  def detach_context(self, ec_id):
    self._rtcout.RTC_TRACE("detach_context(%s)", ec_id)
    _len = len(self._ecOther)

    # ID: 0 - (offset-1) : owned ec
    # ID: offset -       : participating ec
    # owned       ec index = ID
    # participate ec index = ID - offset
    if (long(ec_id) < long(ECOTHER_OFFSET)) or \
          (long(ec_id - ECOTHER_OFFSET) > _len):
      return RTC.BAD_PARAMETER
    
    index = long(ec_id - ECOTHER_OFFSET)

    if CORBA.is_nil(self._ecOther[index]):
      return RTC.BAD_PARAMETER
    
    OpenRTM_aist.CORBA_SeqUtil.erase(self._ecOther, index)

    return RTC.RTC_OK


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC ν
  #
  # RTC 졢Alive ֤ܤ롣
  # RTC ͭνϤǼ¹Ԥ롣
  # Υڥ졼ƤӽФη̤Ȥ onInitialize() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Initialize RTC
  #
  # The RTC has been initialized and entered the Alive state.
  # Any RTC-specific initialization logic should be performed here.
  #
  # @return
  #
  # @endif
  def on_initialize(self):
    self._rtcout.RTC_TRACE("on_initialize()")
    ret = RTC.RTC_ERROR
    try:
      active_config = self._properties.getProperty("active_config")
      if ((active_config is None) or (active_config is "") or (active_config == [])):
        self._configsets.update("default")
      else:
        if self._configsets.haveConfig(active_config):
          self._configsets.update(active_config)
        else:
          self._configsets.update("default")
      ret = self.onInitialize()
    except:
      return RTC.RTC_ERROR

    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC νλ
  #
  # RTC ˴롣
  # RTC ͭνλϤǼ¹Ԥ롣
  # Υڥ졼ƤӽФη̤Ȥ onFinalize() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Finalize RTC
  #
  # The RTC is being destroyed.
  # Any final RTC-specific tear-down logic should be performed here.
  #
  # @return
  #
  # @endif
  def on_finalize(self):
    self._rtcout.RTC_TRACE("on_finalize()")
    ret = RTC.RTC_ERROR
    try:
      ret = self.onFinalize()
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC γ
  #
  # RTC ° ExecutionContext  Stopped ֤ Running ֤
  # ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onStartup() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  # @param ec_id ܤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] StartUp RTC
  #
  # The given execution context, in which the RTC is participating, has 
  # transitioned from Stopped to Running.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_startup(self, ec_id):
    self._rtcout.RTC_TRACE("on_startup(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onStartup(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC 
  #
  # RTC ° ExecutionContext  Running ֤ Stopped ֤
  # ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onShutdown() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  # @param ec_id ܤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] ShutDown RTC
  #
  # The given execution context, in which the RTC is participating, has 
  # transitioned from Running to Stopped.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_shutdown(self, ec_id):
    self._rtcout.RTC_TRACE("on_shutdown(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onShutdown(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC γ
  #
  # ° ExecutionContext  RTC 줿ݤ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onActivated() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  # @param ec_id  ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Activate RTC
  #
  # The RTC has been activated in the given execution context.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_activated(self, ec_id):
    self._rtcout.RTC_TRACE("on_activated(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      self._configsets.update()
      ret = self.onActivated(ec_id)
      self._portAdmin.activatePorts()
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC 
  #
  # ° ExecutionContext  RTC 줿ݤ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onDeactivated() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  # @param ec_id  ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Deactivate RTC
  #
  # The RTC has been deactivated in the given execution context.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_deactivated(self, ec_id):
    self._rtcout.RTC_TRACE("on_deactivated(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      self._portAdmin.deactivatePorts()
      ret = self.onDeactivated(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC Υ顼֤ؤ
  #
  # RTC ° ExecutionContext  Active ֤ Error ֤ܤ
  # ˸ƤӽФ롣
  # Υڥ졼 RTC  Error ֤ܤݤ˰٤ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onAborting() Хåؿ
  # ƤӽФ롣
  #
  # @param self
  # @param ec_id ܤ ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Transition Error State
  #
  # The RTC is transitioning from the Active state to the Error state in some
  # execution context.
  # This callback is invoked only a single time for time that the RTC 
  # transitions into the Error state from another state. This behavior is in 
  # contrast to that of on_error.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_aborting(self, ec_id):
    self._rtcout.RTC_TRACE("on_aborting(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onAborting(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC Υ顼
  #
  # RTC 顼֤ˤݤ˸ƤӽФ롣
  # RTC 顼֤ξˡоݤȤʤ ExecutionContext ExecutionKind 
  # ߥ󥰤ǸƤӽФ롣㤨С
  # - ExecutionKind  PERIODIC ξ硢ܥڥ졼
  #   DataFlowComponentAction::on_execute  on_state_update ؤˡ
  #   ꤵ줿֡ꤵ줿ǸƤӽФ롣
  # - ExecutionKind  EVENT_DRIVEN ξ硢ܥڥ졼
  #   FsmParticipantAction::on_action ƤФ줿ݤˡؤ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onError() ХåؿƤӽ
  # 롣
  #
  # @param self
  # @param ec_id о ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Error Processing of RTC
  #
  # The RTC remains in the Error state.
  # If the RTC is in the Error state relative to some execution context when
  # it would otherwise be invoked from that context (according to the 
  # contexts ExecutionKind), this callback shall be invoked instead. 
  # For example,
  # - If the ExecutionKind is PERIODIC, this operation shall be invoked in 
  #   sorted order at the rate of the context instead of 
  #   DataFlowComponentAction::on_execute and on_state_update.
  # - If the ExecutionKind is EVENT_DRIVEN, this operation shall be invoked 
  #   whenever FsmParticipantAction::on_action would otherwise have been 
  #   invoked.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_error(self, ec_id):
    self._rtcout.RTC_TRACE("on_error(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onError(ec_id)
      self._configsets.update()
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [ComponentAction CORBA interface] RTC Υꥻå
  #
  # Error ֤ˤ RTC ΥꥫХ¹ԤInactive ֤
  # ˸ƤӽФ롣
  # RTC ΥꥫХ Inactive ֤뤬ʳ
  # ˤ Error ֤αޤ롣
  # Υڥ졼ƤӽФη̤Ȥ onReset() ХåؿƤ
  # Ф롣
  #
  # @param self
  # @param ec_id ꥻåо ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [ComponentAction CORBA interface] Resetting RTC
  #
  # The RTC is in the Error state. An attempt is being made to recover it such
  # that it can return to the Inactive state.
  # If the RTC was successfully recovered and can safely return to the
  # Inactive state, this method shall complete with ReturnCode_t::OK. Any
  # other result shall indicate that the RTC should remain in the Error state.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_reset(self, ec_id):
    self._rtcout.RTC_TRACE("on_reset(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onReset(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [DataFlowComponentAction CORBA interface] RTC ()
  #
  # ʲξ֤ݻƤˡꤵ줿Ū˸ƤӽФ롣
  # - RTC  Alive ֤Ǥ롣
  # - ꤵ줿 ExecutionContext  Running ֤Ǥ롣
  # ܥڥ졼ϡTwo-Pass Execution Ǽ¹Ԥ롣
  # Υڥ졼ƤӽФη̤Ȥ onExecute() ХåؿƤ
  # Ф롣
  #
  # 
  # - ꤵ줿 ExecutionContext  ExecutionKind ϡ PERIODIC ǤʤФ
  #   ʤ
  #
  # @param self
  # @param ec_id о ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [DataFlowComponentAction CORBA interface] Primary Periodic 
  #        Operation of RTC
  #
  # This operation will be invoked periodically at the rate of the given
  # execution context as long as the following conditions hold:
  # - The RTC is Active.
  # - The given execution context is Running
  # This callback occurs during the first execution pass.
  #
  # Constraints
  # - The execution context of the given context shall be PERIODIC.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_execute(self, ec_id):
    self._rtcout.RTC_TRACE("on_execute(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onExecute(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [DataFlowComponentAction CORBA interface] RTC ()
  #
  # ʲξ֤ݻƤˡꤵ줿Ū˸ƤӽФ롣
  # - RTC  Alive ֤Ǥ롣
  # - ꤵ줿 ExecutionContext  Running ֤Ǥ롣
  # ܥڥ졼ϡTwo-Pass Execution Ǽ¹Ԥ롣
  # Υڥ졼ƤӽФη̤Ȥ onStateUpdate() Хåؿ
  # ƤӽФ롣
  #
  # 
  # - ꤵ줿 ExecutionContext  ExecutionKind ϡ PERIODIC ǤʤФ
  #   ʤ
  #
  # @param self
  # @param ec_id о ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [DataFlowComponentAction CORBA interface] Secondary Periodic 
  #        Operation of RTC
  #
  # This operation will be invoked periodically at the rate of the given
  # execution context as long as the following conditions hold:
  # - The RTC is Active.
  # - The given execution context is Running
  # This callback occurs during the second execution pass.
  #
  # Constraints
  # - The execution context of the given context shall be PERIODIC.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_state_update(self, ec_id):
    self._rtcout.RTC_TRACE("on_state_update(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onStateUpdate(ec_id)
      self._configsets.update()
    except:
      return RTC.RTC_ERROR
    
    return ret


  ##
  # @if jp
  #
  # @brief [DataFlowComponentAction CORBA interface] ¹Լѹ
  #
  # ܥڥ졼ϡExecutionContext μ¹Լѹ줿ȤΤ
  # ݤ˸ƤӽФ롣
  # Υڥ졼ƤӽФη̤Ȥ onRateChanged() Хåؿ
  # ƤӽФ롣
  #
  # 
  # - ꤵ줿 ExecutionContext  ExecutionKind ϡ PERIODIC ǤʤФ
  #   ʤ
  #
  # @param self
  # @param ec_id о ExecutionContext  ID
  #
  # @return ReturnCode_t Υ꥿󥳡
  #
  # @else
  #
  # @brief [DataFlowComponentAction CORBA interface] Notify rate chenged
  #
  # This operation is a notification that the rate of the indicated execution 
  # context has changed.
  #
  # Constraints
  # - The execution context of the given context shall be PERIODIC.
  #
  # @param ec_id
  #
  # @return
  #
  # @endif
  def on_rate_changed(self, ec_id):
    self._rtcout.RTC_TRACE("on_rate_changed(%s)", ec_id)
    ret = RTC.RTC_ERROR
    try:
      ret = self.onRateChanged(ec_id)
    except:
      return RTC.RTC_ERROR
    
    return ret


  #============================================================
  # SDOPackage::SdoSystemElement
  #============================================================

  ##
  # @if jp
  # 
  # @brief [SDO interface] Organization ꥹȤμ 
  #
  # SDOSystemElement 0Ĥ⤷Ϥʾ Organization ͭ뤳Ȥ
  # 롣 SDOSystemElement 1İʾ Organization ͭƤ
  # ˤϡΥڥ졼Ͻͭ Organization ΥꥹȤ֤
  # ⤷OrganizationĤͭƤʤжΥꥹȤ֤
  #
  # @param self
  #
  # @return ͭƤ Organization ꥹ
  #
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting Organizations
  #
  # SDOSystemElement can be the owner of zero or more organizations.
  # If the SDOSystemElement owns one or more Organizations, this operation
  # returns the list of Organizations that the SDOSystemElement owns.
  # If it does not own any Organization, it returns empty list.
  #
  # @return Owned Organization List
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable if the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError if the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::OrganizationList* get_owned_organizations()
  def get_owned_organizations(self):
    self._rtcout.RTC_TRACE("get_owned_organizations()")
    try:
      return self._sdoOwnedOrganizations
    except:
      raise SDOPackage.NotAvailable("NotAvailable: get_owned_organizations")

    return []


  #============================================================
  # SDOPackage::SDO
  #============================================================

  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO ID μ
  #
  # SDO ID ֤ڥ졼
  # Υڥ졼ϰʲη㳰ȯ롣
  #
  # @param self
  # 
  # @return    ꥽ǡǥƤ SDO  ID
  # 
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting SDO ID
  #
  # This operation returns id of the SDO.
  # This operation throws SDOException with one of the following types.
  #
  # @return    id of the SDO defined in the resource data model.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable if the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError if the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual char* get_sdo_id()
  def get_sdo_id(self):
    self._rtcout.RTC_TRACE("get_sdo_id()")
    try:
      return self._profile.instance_name
    except:
      raise SDOPackage.InternalError("get_sdo_id()")


  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO פμ
  # 
  # SDO Type ֤ڥ졼
  # Υڥ졼ϰʲη㳰ȯ롣
  #
  # @param self
  #
  # @return    ꥽ǡǥƤ SDO  Type
  #
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting SDO type
  #
  # This operation returns sdoType of the SDO.
  # This operation throws SDOException with one of the following types.
  #
  # @return    Type of the SDO defined in the resource data model.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable if the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError if the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual char* get_sdo_type()
  def get_sdo_type(self):
    self._rtcout.RTC_TRACE("get_sdo_type()")
    try:
      return self._profile.description
    except:
      raise SDOPackage.InternalError("get_sdo_type()")
    return ""


  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO DeviceProfile ꥹȤμ 
  #
  # SDO  DeviceProfile ֤ڥ졼 SDO ϡɥǥХ
  # ˴ϢդƤʤˤϡ DeviceProfile ֤롣
  # Υڥ졼ϰʲη㳰ȯ롣
  #
  # @param self
  #
  # @return    SDO DeviceProfile
  #
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting SDO DeviceProfile
  #
  # This operation returns the DeviceProfile of the SDO. If the SDO does not
  # represent any hardware device, then a DeviceProfile with empty values
  # are returned.
  # This operation throws SDOException with one of the following types.
  #
  # @return    The DeviceProfile of the SDO.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable if the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError if the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::DeviceProfile* get_device_profile()
  def get_device_profile(self):
    self._rtcout.RTC_TRACE("get_device_profile()")
    try:
      dprofile = SDOPackage.DeviceProfile(self._SdoConfigImpl.getDeviceProfile().device_type,
                                          self._SdoConfigImpl.getDeviceProfile().manufacturer,
                                          self._SdoConfigImpl.getDeviceProfile().model,
                                          self._SdoConfigImpl.getDeviceProfile().version,
                                          self._SdoConfigImpl.getDeviceProfile().properties)
      return dprofile
    except:
      raise SDOPackage.InternalError("get_device_profile()")

    return SDOPackage.DeviceProfile("","","","",[])


  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO ServiceProfile μ 
  #
  # SDO ͭƤ Service  ServiceProfile ֤ڥ졼
  # SDO ӥĤͭƤʤˤϡΥꥹȤ֤
  # Υڥ졼ϰʲη㳰ȯ롣
  #
  # @param self
  # 
  # @return    SDO 󶡤Ƥ Service  ServiceProfile
  # 
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting SDO ServiceProfile
  # 
  # This operation returns a list of ServiceProfiles that the SDO has.
  # If the SDO does not provide any service, then an empty list is returned.
  # This operation throws SDOException with one of the following types.
  # 
  # @return    List of ServiceProfiles of all the services the SDO is
  #            providing.
  # 
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable if the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError if the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::ServiceProfileList* get_service_profiles()
  def get_service_profiles(self):
    self._rtcout.RTC_TRACE("get_service_profiles()")
    self._sdoSvcProfiles = self._SdoConfigImpl.getServiceProfiles()
    try:
      return self._sdoSvcProfiles
    except:
      raise SDOPackage.InternalError("get_service_profiles()")

    return []


  ##
  # @if jp
  # 
  # @brief [SDO interface] ServiceProfileμ 
  #
  #  "id" ǻꤵ줿̾Υӥ ServiceProfile ֤
  # 
  # @param     self
  # @param     _id SDO Service  ServiceProfile ˴Ϣդ줿̻ҡ
  # 
  # @return    ꤵ줿 SDO Service  ServiceProfile
  # 
  # @exception InvalidParameter "id" ǻꤷ ServiceProfile ¸ߤʤ
  #                             "id"  null
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting Organizations
  #
  # This operation returns the ServiceProfile that is specified by the
  # argument "id."
  # 
  # @param     _id The identifier referring to one of the ServiceProfiles.
  # 
  # @return    The profile of the specified service.
  # 
  # @exception InvalidParameter if the ServiceProfile that is specified by 
  #                             the argument 'id' does not exist or if 'id'
  #                             is 'null.'
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable If the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError If the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::ServiceProfile* get_service_profile(const char* id)
  def get_service_profile(self, _id):
    self._rtcout.RTC_TRACE("get_service_profile(%s)", _id)
    self._sdoSvcProfiles = self._SdoConfigImpl.getServiceProfiles()
    if not _id:
      raise SDOPackage.InvalidParameter("get_service_profile(): Empty name.")

    try:
      index = OpenRTM_aist.CORBA_SeqUtil.find(self._sdoSvcProfiles, self.svc_name(_id))

      if index < 0:
        raise SDOPackage.InvalidParameter("get_service_profile(): Not found")

      return self._sdoSvcProfiles[index]
    except:
      raise SDOPackage.InternalError("get_service_profile()")

    return SDOPackage.ServiceProfile("", "", [], None)


  ##
  # @if jp
  # 
  # @brief [SDO interface] ꤵ줿 SDO Service μ
  #
  # Υڥ졼ϰ "id" ǻꤵ줿̾ˤäƶ̤
  # SDO  Service ؤΥ֥ȻȤ֤ SDO ˤ󶡤
  # Service Ϥ줾դμ̻Ҥˤ̤롣
  #
  # @param self
  # @param _id SDO Service ˴Ϣդ줿̻ҡ
  #
  # @return ׵ᤵ줿 SDO Service ؤλȡ
  #
  # 
  # @exception InvalidParameter "id" ǻꤷ ServiceProfile ¸ߤʤ
  #                             "id"  null
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting specified SDO Service's reference
  #
  # This operation returns an object implementing an SDO's service that
  # is identified by the identifier specified as an argument. Different
  # services provided by an SDO are distinguished with different
  # identifiers. See OMG SDO specification Section 2.2.8, "ServiceProfile,"
  # on page 2-12 for more details.
  #
  # @param _id The identifier referring to one of the SDO Service
  # @return The object implementing the requested service.
  # @exception InvalidParameter if argument id is null, or if the 
  #                             ServiceProfile that is specified by argument
  #                            id does not exist.
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable If the target SDO is reachable but cannot
  #                         respond.
  # @exception InternalError If the target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::SDOService_ptr get_sdo_service(const char* id)
  def get_sdo_service(self, _id):
    self._rtcout.RTC_TRACE("get_sdo_service(%s)", _id)
    self._sdoSvcProfiles = self._SdoConfigImpl.getServiceProfiles()

    if not _id:
      raise SDOPackage.InvalidParameter("get_service(): Empty name.")

    index = OpenRTM_aist.CORBA_SeqUtil.find(self._sdoSvcProfiles, self.svc_name(_id))

    if index < 0:
      raise SDOPackage.InvalidParameter("get_service(): Not found")

    try:
      return self._sdoSvcProfiles[index].service
    except:
      raise SDOPackage.InternalError("get_service()")
    return SDOPackage.SDOService._nil


  ##
  # @if jp
  # 
  # @brief [SDO interface] Configuration ֥Ȥμ 
  #
  # Υڥ졼 Configuration interface ؤλȤ֤
  # Configuration interface ϳ SDO 뤿Υ󥿡ե
  # ҤȤĤǤ롣Υ󥿡ե DeviceProfile, ServiceProfile,
  # Organization 줿 SDO °ͤꤹ뤿˻Ѥ롣
  # Configuration 󥿡եξܺ٤ˤĤƤϡOMG SDO specification
  #  2.3.5, p.2-24 򻲾ȤΤȡ
  #
  # @param self
  #
  # @return SDO  Configuration 󥿡եؤλ
  #
  # @exception InterfaceNotImplemented SDOConfiguration󥿡ե
  #                                    ʤ
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Getting Configuration object
  #
  # This operation returns an object implementing the Configuration
  # interface. The Configuration interface is one of the interfaces that
  # each SDO maintains. The interface is used to configure the attributes
  # defined in DeviceProfile, ServiceProfile, and Organization.
  # See OMG SDO specification Section 2.3.5, "Configuration Interface,"
  # on page 2-24 for more details about the Configuration interface.
  #
  # @return The Configuration interface of an SDO.
  #
  # @exception InterfaceNotImplemented The target SDO has no Configuration
  #                                    interface.
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable The target SDO is reachable but cannot respond.
  # @exception InternalError The target SDO cannot execute the operation
  #                          completely due to some internal error.
  # @endif
  # virtual SDOPackage::Configuration_ptr get_configuration()
  def get_configuration(self):
    self._rtcout.RTC_TRACE("get_configuration()")
    if self._SdoConfig is None:
      raise SODPackage.InterfaceNotImplemented("InterfaceNotImplemented: get_configuration")
    try:
      return self._SdoConfig
    except:
      raise SDOPackage.InternalError("get_configuration()")
    return SDOPackage.Configuration._nil


  ##
  # @if jp
  # 
  # @brief [SDO interface] Monitoring ֥Ȥμ 
  #
  # Υڥ졼 Monitoring interface ؤλȤ֤
  # Monitoring interface  SDO 륤󥿡եΰĤǤ롣
  # Υ󥿡ե SDO Υץѥƥ˥󥰤뤿
  # Ѥ롣
  # Monitoring interface ξܺ٤ˤĤƤ OMG SDO specification 
  # 2.3.7 "Monitoring Interface" p.2-35 򻲾ȤΤȡ
  #
  # @param self
  #
  # @return SDO  Monitoring interface ؤλ
  #
  # @exception InterfaceNotImplemented SDOConfiguration󥿡ե
  #                                    ʤ
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Get Monitoring object
  #
  # This operation returns an object implementing the Monitoring interface.
  # The Monitoring interface is one of the interfaces that each SDO
  # maintains. The interface is used to monitor the properties of an SDO.
  # See OMG SDO specification Section 2.3.7, "Monitoring Interface," on
  # page 2-35 for more details about the Monitoring interface.
  #
  # @return The Monitoring interface of an SDO.
  #
  # @exception InterfaceNotImplemented The target SDO has no Configuration
  #                                    interface.
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable The target SDO is reachable but cannot respond.
  # @exception InternalError The target SDO cannot execute the operation
  #                          completely due to some internal error.
  # @endif
  # virtual SDOPackage::Monitoring_ptr get_monitoring()
  def get_monitoring(self):
    self._rtcout.RTC_TRACE("get_monitoring()")
    raise SDOPackage.InterfaceNotImplemented("Exception: get_monitoring")
    return SDOPackage.Monitoring._nil


  ##
  # @if jp
  # 
  # @brief [SDO interface] Organization ꥹȤμ 
  #
  # SDO 0İʾ Organization (ȿ)˽°뤳ȤǤ롣 ⤷ SDO 
  # 1İʾ Organization ˽°Ƥ硢Υڥ졼Ͻ°
  # Organization ΥꥹȤ֤SDO  ɤ Organization ˤ°Ƥʤ
  # ˤϡΥꥹȤ֤롣
  #
  # @param self
  #
  # @return SDO ° Organization Υꥹȡ
  #
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  # @else
  #
  # @brief [SDO interface] Getting Organizations
  #
  # An SDO belongs to zero or more organizations. If the SDO belongs to one
  # or more organizations, this operation returns the list of organizations
  # that the SDO belongs to. An empty list is returned if the SDO does not
  # belong to any Organizations.
  #
  # @return The list of Organizations that the SDO belong to.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable The target SDO is reachable but cannot respond.
  # @exception InternalError The target SDO cannot execute the operation
  #                          completely due to some internal error.
  # @endif
  # virtual SDOPackage::OrganizationList* get_organizations()
  def get_organizations(self):
    self._rtcout.RTC_TRACE("get_organizations()")
    self._sdoOrganizations = self._SdoConfigImpl.getOrganizations()
    try:
      return self._sdoOrganizations
    except:
      raise SDOPackage.InternalError("get_organizations()")
    return []


  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO Status ꥹȤμ 
  #
  # Υڥ졼 SDO Υơɽ NVList ֤
  #
  # @param self
  #
  # @return SDO Υơ
  #
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InternalError Ū顼ȯ
  #
  # @else
  #
  # @brief [SDO interface] Get SDO Status
  #
  # This operation returns an NVlist describing the status of an SDO.
  #
  # @return The actual status of an SDO.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable The target SDO is reachable but cannot respond.
  # @exception InternalError The target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  # @endif
  # virtual SDOPackage::NVList* get_status_list()
  def get_status_list(self):
    self._rtcout.RTC_TRACE("get_status_list()")
    try:
      return self._sdoStatus
    except:
      raise SDOPackage.InternalError("get_status_list()")
    return []


  ##
  # @if jp
  # 
  # @brief [SDO interface] SDO Status μ 
  #
  # This operation returns the value of the specified status parameter.
  #
  # @param self
  # @param name SDO Υơѥ᡼
  # 
  # @return ꤵ줿ѥ᡼Υơ͡
  # 
  # @exception SDONotExists åȤSDO¸ߤʤ(㳰ϡCORBAɸ
  #                         ƥ㳰OBJECT_NOT_EXIST˥ޥåԥ󥰤)
  # @exception NotAvailable SDO¸ߤ뤬ʤ
  # @exception InvalidParameter  "name"  null 뤤¸ߤʤ
  # @exception InternalError Ū顼ȯ
  # @else
  #
  # @brief [SDO interface] Get SDO Status
  #
  # @param name One of the parameters defining the "status" of an SDO.
  #
  # @return The value of the specified status parameter.
  #
  # @exception SDONotExists if the target SDO does not exist.(This exception 
  #                         is mapped to CORBA standard system exception
  #                         OBJECT_NOT_EXIST.)
  # @exception NotAvailable The target SDO is reachable but cannot respond.
  # @exception InvalidParameter The parameter defined by "name" is null or
  #                             does not exist.
  # @exception InternalError The target SDO cannot execute the operation
  #                          completely due to some internal error.
  #
  #
  # @endif
  # virtual CORBA::Any* get_status(const char* name)
  def get_status(self, name):
    self._rtcout.RTC_TRACE("get_status(%s)", name)
    index = OpenRTM_aist.CORBA_SeqUtil.find(self._sdoStatus, self.nv_name(name))
    if index < 0:
      raise SDOPackage.InvalidParameter("get_status(): Not found")

    try:
      return any.to_any(self._sdoStatus[index].value)
    except:
      raise SDOPackage.InternalError("get_status()")
    return any.to_any("")


  #============================================================
  # Local interfaces
  #============================================================

  ##
  # @if jp
  #
  # @brief [local interface] 󥹥̾μ
  # 
  # ComponentProfile ꤵ줿󥹥֤̾
  #
  # @param self
  # 
  # @return 󥹥̾
  # 
  # @else
  # 
  # @endif
  # const char* getInstanceName()
  def getInstanceName(self):
    self._rtcout.RTC_TRACE("getInstanceName()")
    return self._profile.instance_name


  ##
  # @if jp
  #
  # @brief [local interface] 󥹥̾
  # 
  # ComponentProfile ˻ꤵ줿󥹥̾ꤹ롣
  #
  # @param self
  # 
  # @param instance_name 󥹥̾
  # 
  # @else
  # 
  # @endif
  # void setInstanceName(const char* instance_name);
  def setInstanceName(self, instance_name):
    self._rtcout.RTC_TRACE("setInstanceName(%s)", instance_name)
    self._properties.setProperty("instance_name",instance_name)
    self._profile.instance_name = self._properties.getProperty("instance_name")


  ##
  # @if jp
  #
  # @brief [local interface] ̾μ
  # 
  # ComponentProfile ꤵ줿֤̾
  #
  # @param self
  # 
  # @return ̾
  # 
  # @else
  # 
  # @endif
  # const char* getTypeName()
  def getTypeName(self):
    self._rtcout.RTC_TRACE("getTypeName()")
    return self._profile.type_name


  ##
  # @if jp
  #
  # @brief [local interface] Description μ
  # 
  # ComponentProfile ꤵ줿 Description ֤
  #
  # @param self
  # 
  # @return Description
  # 
  # @else
  # 
  # @endif
  # const char* getDescription()
  def getDescription(self):
    self._rtcout.RTC_TRACE("getDescription()")
    return self._profile.description


  ##
  # @if jp
  #
  # @brief [local interface] Сμ
  # 
  # ComponentProfile ꤵ줿С֤
  #
  # @param self
  # 
  # @return С
  # 
  # @else
  # 
  # @endif
  # const char* getVersion()
  def getVersion(self):
    self._rtcout.RTC_TRACE("getVersion()")
    return self._profile.version


  ##
  # @if jp
  #
  # @brief [local interface] ٥μ
  # 
  # ComponentProfile ꤵ줿٥֤
  #
  # @param self
  # 
  # @return ٥
  # 
  # @else
  # 
  # @endif
  # const char* getVendor()
  def getVendor(self):
    self._rtcout.RTC_TRACE("getVendor()")
    return self._profile.vendor


  ##
  # @if jp
  #
  # @brief [local interface] ƥμ
  # 
  # ComponentProfile ꤵ줿ƥ֤
  #
  # @param self
  # 
  # @return ƥ
  # 
  # @else
  # 
  # @endif
  # const char* getCategory()
  def getCategory(self):
    self._rtcout.RTC_TRACE("getCategory()")
    return self._profile.category


  ##
  # @if jp
  #
  # @brief [local interface] Naming Server μ
  # 
  # ꤵ줿 Naming Server ֤
  #
  # @param self
  # 
  # @return Naming Server ꥹ
  # 
  # @else
  # 
  # @endif
  # std::vector<std::string> getNamingNames();
  def getNamingNames(self):
    self._rtcout.RTC_TRACE("getNamingNames()")
    return string.split(self._properties.getProperty("naming.names"), ",")


  ##
  # @if jp
  #
  # @brief [local interface] ֥ȥե󥹤
  # 
  # RTC  CORBA ֥ȥե󥹤ꤹ롣
  # 
  # @param self
  # @param rtobj ֥ȥե
  # 
  # @else
  # 
  # @endif
  # void setObjRef(const RTObject_ptr rtobj);
  def setObjRef(self, rtobj):
    self._rtcout.RTC_TRACE("setObjRef()")
    self._objref = rtobj
    return


  ##
  # @if jp
  #
  # @brief [local interface] ֥ȥե󥹤μ
  # 
  # ꤵ줿 CORBA ֥ȥե󥹤롣
  # 
  # @param self
  # 
  # @return ֥ȥե
  # 
  # @else
  # 
  # @endif
  # RTObject_ptr getObjRef() const;
  def getObjRef(self):
    self._rtcout.RTC_TRACE("getObjRef()")
    return self._objref


  ##
  # @if jp
  # 
  # @brief [local interface] RTC Υץѥƥꤹ
  #
  # RTC ݻ٤ץѥƥꤹ롣Ϳץѥƥϡ
  # ComponentProfile ꤵ٤ʤФʤʤ
  # Υڥ졼̾ RTC ݤ Manager 
  # ƤФ뤳ȤտޤƤ롣
  # 
  # @param self
  # @param prop RTC Υץѥƥ
  #
  # @else
  #
  # @brief [local interface] Set RTC property
  #
  # This operation sets the properties to the RTC. The given property
  # values should include information for ComponentProfile.
  # Generally, this operation is designed to be called from Manager, when
  # RTC is initialized
  #
  # @param prop Property for RTC.
  #
  # @endif
  # void setProperties(const coil::Properties& prop);
  def setProperties(self, prop):
    self._rtcout.RTC_TRACE("setProperties()")
    self._properties.mergeProperties(prop)
    self._profile.instance_name = self._properties.getProperty("instance_name")
    self._profile.type_name     = self._properties.getProperty("type_name")
    self._profile.description   = self._properties.getProperty("description")
    self._profile.version       = self._properties.getProperty("version")
    self._profile.vendor        = self._properties.getProperty("vendor")
    self._profile.category      = self._properties.getProperty("category")


  ##
  # @if jp
  # 
  # @brief [local interface] RTC Υץѥƥ
  #
  # RTC ݻƤץѥƥ֤
  # RTCץѥƥʤ϶Υץѥƥ֤롣
  # 
  # @param self
  # 
  # @return RTC Υץѥƥ
  #
  # @else
  #
  # @brief [local interface] Get RTC property
  #
  # This operation returns the properties of the RTC.
  # Empty property would be returned, if RTC has no property.
  #
  # @return Property for RTC.
  #
  # @endif
  # coil::Properties& getProperties();
  def getProperties(self):
    self._rtcout.RTC_TRACE("getProperties()")
    return self._properties


  ##
  # @if jp
  #
  # @brief ե졼ѥ᡼
  # 
  # ե졼ѥ᡼ѿХɤ
  # \<VarType\>Ȥƥե졼ѥ᡼Υǡꤹ롣
  #
  # @param self
  # @param param_name ե졼ѥ᡼̾
  # @param var ե졼ѥ᡼Ǽѿ
  # @param def_val ե졼ѥ᡼ǥե
  # @param trans ʸѴѴؿ(ǥե:None)
  #
  # @return (:true꼺:false)
  # 
  # @else
  #
  # @endif
  #  template <typename VarType>
  #  bool bindParameter(const char* param_name, VarType& var,
  #                     const char* def_val,
  #                     bool (*trans)(VarType&, const char*) = coil::stringTo)
  def bindParameter(self, param_name, var,
                    def_val, trans=None):
    self._rtcout.RTC_TRACE("bindParameter()")
    if trans is None:
      _trans = OpenRTM_aist.stringTo
    else:
      _trans = trans
    self._configsets.bindParameter(param_name, var, def_val, _trans)
    return True


  ##
  # @if jp
  #
  # @brief ե졼ѥ᡼ι(ID)
  # 
  # ꤷIDΥե졼󥻥åȤꤷͤǡ
  # ե졼ѥ᡼ͤ򹹿
  #
  # @param self
  # @param config_set оݤΥե졼󥻥åID
  # 
  # @else
  #
  # @endif
  # void updateParameters(const char* config_set);
  def updateParameters(self, config_set):
    self._rtcout.RTC_TRACE("updateParameters(%s)", config_set)
    self._configsets.update(config_set)
    return


  ##
  # @if jp
  # 
  # @brief [local interface] Port Ͽ
  #
  # RTC ݻPortϿ롣
  # Port 饢ǽˤ뤿ˤϡΥڥ졼ˤ
  # ϿƤʤФʤʤϿ Port Ϥ RTC ˤ
  # PortProfile.name ˤ̤롣äơPort  RTC ˤơ
  # ˡ PortProfile.name ʤФʤʤ
  # Ͽ줿 Port Ŭڤ˥ƥֲ줿塢λȤ
  # ֥ȻȤꥹ¸롣
  # 
  # @param self
  # @param port RTC Ͽ Port
  # @param port_type if port is PortBase, port_type is None,
  #                  if port is PortService, port_type is True
  #
  # @else
  #
  # @brief [local interface] Register Port
  #
  # This operation registers a Port to be held by this RTC.
  # In order to enable access to the Port from outside of RTC, the Port
  # must be registered by this operation. The Port that is registered by
  # this operation would be identified by PortProfile.name in the inside of
  # RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
  # The registering Port would be activated properly, and the reference
  # and the object reference would be stored in lists in RTC.
  #
  # @param port Port which is registered in the RTC
  #
  # @endif
  # void registerPort(PortBase& port);
  def registerPort(self, port):
    self._rtcout.RTC_TRACE("registerPort()")
    self._portAdmin.registerPort(port)
    port.setOwner(self.getObjRef())
    return

  # void registerPort(PortService_ptr port);
  def registerPortByReference(self, port_ref):
    self._rtcout.RTC_TRACE("registerPortByReference()")
    self._portAdmin.registerPortByReference(port_ref)
    return

  ##
  # @if jp
  # 
  # @brief [local interface] DataInPort Ͽ
  #
  # RTC ݻ DataInPort Ͽ롣
  # Port Υץѥƥ˥ǡݡȤǤ뤳("port.dataport")
  # TCPѤ뤳("tcp_any")ꤹȤȤˡ DataInPort 
  # 󥹥󥹤Ͽ롣
  # 
  # @param self
  # @param name port ̾
  # @param inport Ͽо DataInPort
  #
  # @else
  #
  # @endif
  def registerInPort(self, name, inport):
    self._rtcout.RTC_TRACE("registerInPort(%s)", name)
    if self._properties.hasKey("port.inport"):
      inport.properties().mergeProperties(self._properties.getNode("port.inport"))

    propkey = "port.dataport."
    propkey += name
    if self._properties.hasKey(propkey):
      inport.properties().mergeProperties(self._properties.getNode(propkey))

    inport.init()
    self.registerPort(inport)
    return


  ##
  # @if jp
  # 
  # @brief [local interface] DataOutPort Ͽ
  #
  # RTC ݻ DataOutPor tϿ롣
  # Port Υץѥƥ˥ǡݡȤǤ뤳("port.dataport")
  # TCPѤ뤳("tcp_any")ꤹȤȤˡ DataOutPort 
  # 󥹥󥹤Ͽ롣
  # 
  # @param self
  # @param name port ̾
  # @param outport Ͽо DataInPort
  #
  # @else
  #
  # @endif
  # void registerOutPort(const char* name, OutPortBase& outport);
  def registerOutPort(self, name, outport):
    self._rtcout.RTC_TRACE("registerOutPort(%s)", name)
    propkey = "port.dataport."
    propkey += name
    prop = self._properties.getNode(propkey)
    prop.mergeProperties(self._properties.getNode("port.outport.dataport"))
    outport.properties().mergeProperties(self._properties.getNode(propkey))
    self.registerPort(outport)
    return


  ##
  # @if jp
  # 
  # @brief [local interface] Port Ͽ
  #
  # RTC ݻPortϿ롣
  # 
  # @param self
  # @param port о Port
  #
  # @else
  #
  # @brief [local interface] Unregister Port
  #
  # This operation unregisters a Port to be held by this RTC.
  #
  # @param port Port which is unregistered in the RTC
  #
  # @endif
  # void RTObject_impl::deletePort(PortBase& port)
  def deletePort(self, port):
    self._rtcout.RTC_TRACE("deletePort()")
    self._portAdmin.deletePort(port)
    return


  ##
  # @if jp
  # 
  # @brief [local interface] ̾ˤ Port Ͽ
  #
  # ̾Τꤷ RTC ݻPortϿ롣
  # 
  # @param self
  # @param port_name о Port ̾
  #
  # @else
  #
  # @endif
  def deletePortByName(self, port_name):
    self._rtcout.RTC_TRACE("deletePortByName()")
    self._portAdmin.deletePortByName(port_name)
    return


  ##
  # @if jp
  #
  # @brief  Port Ͽ
  #
  # RTC ݻƤ Port 롣
  # 
  # @param self
  #
  # @else
  #
  # @brief Unregister the All Portse
  #
  # This operation deactivates the all Port and deletes the all Port's
  # registrations in the RTC..
  #
  # @endif
  def finalizePorts(self):
    self._rtcout.RTC_TRACE("finalizePorts()")
    self._portAdmin.finalizePorts()
    return


  def finalizeContexts(self):
    self._rtcout.RTC_TRACE("finalizeContexts()")
    for i in range(len(self._eclist)):
      self._eclist[i].stop()
      #self._poa.deactivate_object(self._poa.servant_to_id(self._eclist[i]))
      self._default_POA().deactivate_object(self._default_POA().servant_to_id(self._eclist[i]))
      del self._eclist[i]


  ##
  # @if jp
  #
  # @brief RTC λ
  #
  # RTC νλ¹Ԥ롣
  # ݻƤ Port ϿȤȤˡ CORBA ֥
  # RTC λ롣
  # 
  # @param self
  #
  # @else
  #
  # @endif
  def shutdown(self):
    self._rtcout.RTC_TRACE("shutdown()")
    try:
      self.finalizePorts()
      self.finalizeContexts()
      self._poa.deactivate_object(self._poa.servant_to_id(self._SdoConfigImpl))
      self._poa.deactivate_object(self._poa.servant_to_id(self))
    except:
      #traceback.print_exception(*sys.exc_info())
      self._rtcout.RTC_ERROR(sys.exc_info()[0])

    if self._manager:
      self._manager.cleanupComponent(self)
      
    return



  ##
  # @if jp
  # @class svc_name
  # @brief SDOService ΥץեꥹȤidǥ뤿
  # ե󥯥饹
  # @else
  #
  # @endif
  class svc_name:
    def __init__(self, _id):
      self._id= _id

    def __call__(self, prof):
      return self._id == prof.id


  #------------------------------------------------------------
  # Functor
  #------------------------------------------------------------

  ##
  # @if jp
  # @class nv_name
  # @brief NVList ѥե󥯥
  # @else
  #
  # @endif
  class nv_name:
    def __init__(self, _name):
      self._name = _name

    def __call__(self, nv):
      return self._name == nv.name


  ##
  # @if jp
  # @class ec_find
  # @brief ExecutionContext ѥե󥯥
  # @else
  #
  # @endif
  class ec_find:
    def __init__(self, _ec):
      self._ec = _ec

    def __call__(self, ecs):
      try:
        ec = ecs._narrow(RTC.ExecutionContext)
        return self._ec._is_equivalent(ec)
      except:
        return False

      return False


  ##
  # @if jp
  # @class ec_copy
  # @brief ExecutionContext Copyѥե󥯥
  # @else
  #
  # @endif
  class ec_copy:
    def __init__(self, eclist):
      self._eclist = eclist

    def __call__(self, ecs):
      self._eclist.append(ecs)


  ##
  # @if jp
  # @class deactivate_comps
  # @brief RTC ѥե󥯥
  # @else
  #
  # @endif
  class deactivate_comps:
    def __init__(self, comp):
      self._comp = comp

    def __call__(self, ec):
      try:
        ec.deactivate_component(self._comp)
      except:
        pass


# RtcBase = RTObject_impl
