#!/usr/bin/env/python
# -*- coding: euc-jp -*-

##
# @file Timer.py
# @brief Timer class
# @date $Date: $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2007-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.


import time
import threading

import OpenRTM_aist


##
# @if jp
# @class Timer
# @brief Timer饹
# 
# Ͽ줿ꥹʡΥХåؿꤵ줿Ū˸ƤӽФ
#
# @since 0.4.0
#
# @else
#
# @endif
class Timer:
  """
  """



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  # 
  # 󥹥ȥ饯
  #
  # @param self
  # @param interval ޵ư
  #
  # @else
  #
  # @endif
  def __init__(self, interval):
    self._interval = interval
    self._running  = False
    self._runningMutex = threading.RLock()
    self._tasks = []
    self._taskMutex = threading.RLock()
    self._thread = threading.Thread(target=self.run)


  def __del__(self):
    guard = OpenRTM_aist.ScopedLock(self._runningMutex)
    self._running = False
    self._thread.join()

  ##
  # @if jp
  # @brief Timer ¹
  #
  # Timer ѿåɤŪϿ줿ꥹʡΥ᥽åɤƤӽФ
  #
  # @param self
  #
  # @else
  #
  # @endif
  def run(self):
    while self._running:
      if self._interval.tv_sec != 0:
        time.sleep(self._interval.tv_sec)
      time.sleep(self._interval.tv_usec/1000000.0)
      self.invoke()
    return 0


  ##
  # @if jp
  # @brief Timer 
  #
  # Timer ѿåɤ򳫻Ϥ롣
  #
  # @param self
  #
  # @else
  #
  # @endif
  def start(self):
    guard = OpenRTM_aist.ScopedLock(self._runningMutex)
    if not self._running:
      self._running = True
      self._thread.start()


  ##
  # @if jp
  # @brief Timer 
  #
  # @param self
  #
  # Timer ߤ롣
  #
  # @else
  #
  # @endif
  def stop(self):
    guard = OpenRTM_aist.ScopedLock(self._runningMutex)
    self._running = False


  ##
  # @if jp
  # @brief Timer ¹
  #
  # @param self
  #
  # Ͽ줿ƥꥹʤεưԤ֤饿޵ư򸺻롣
  # ưԤ֤Ȥʤäꥹʤ¸ߤϡ
  # ХåؿƤӽФ
  #
  # @else
  #
  # @endif
  def invoke(self):
    for i in range(len(self._tasks)):
      self._tasks[i].remains = self._tasks[i].remains - self._interval
      if self._tasks[i].remains.sign() <= 0.0:
        self._tasks[i].listener.invoke()
        self._tasks[i].remains = self._tasks[i].period


  ##
  # @if jp
  # @brief ꥹʡϿ
  #
  #  Timer 鵯ư륳ХåؿѤΥꥹʡưꤷ
  # Ͽ롣
  # ƱꥹʡϿѤߤξϡꥹʡεưꤷͤ
  # 롣
  #
  # @param self
  # @param listener Ͽоݥꥹʡ
  # @param tm ꥹʡư
  #
  # @return Ͽꥹʡ
  #
  # @else
  #
  # @endif
  # ListenerId registerListener(ListenerBase* listener, TimeValue tm);
  def registerListener(self, listener, tm):
    guard = OpenRTM_aist.ScopedLock(self._taskMutex)
    for i in range(len(self._tasks)):
      if self._tasks[i].listener == listener:
        self._tasks[i].period = tm
        self._tasks[i].remains = tm
        return listener
    self._tasks.append(self.Task(listener, tm))
    return listener


  ##
  # @if jp
  # @brief ꥹʡϿ
  #
  # Хåоݥ֥ȡХåоݥ᥽åɤӵư
  # ꤷƥꥹʡϿ롣
  #
  # @param self
  # @param obj Хåоݥ֥
  # @param cbf Хåоݥ᥽å
  # @param tm ꥹʡư
  #
  # @return Ͽꥹʡ
  #
  # @else
  #
  # @endif
  #  template <class ListenerClass>
  #  ListenerId registerListenerObj(ListenerClass* obj,
  #                                 void (ListenerClass::*cbf)(),
  #                                 TimeValue tm)
  def registerListenerObj(self, obj, cbf, tm):
    return self.registerListener(OpenRTM_aist.ListenerObject(obj, cbf), tm)


  ##
  # @if jp
  # @brief ꥹʡϿ
  #
  # Хåоݥ᥽åɤȵưꤷƥꥹʡϿ롣
  #
  # @param self
  # @param cbf Хåоݥ᥽å
  # @param tm ꥹʡư
  #
  # @return Ͽꥹʡ
  #
  # @else
  #
  # @endif
  # ListenerId registerListenerFunc(void (*cbf)(), TimeValue tm)
  def registerListenerFunc(self, cbf, tm):
    return self.registerListener(OpenRTM_aist.ListenerFunc(cbf), tm)


  ##
  # @if jp
  # @brief ꥹʡϿ
  #
  # ꤷIDΥꥹʡϿ롣
  # ꤷIDΥꥹʡ̤Ͽξ硢false ֤
  #
  # @param self
  # @param id ϿоݥꥹʡID
  #
  # @return Ͽ
  #
  # @else
  #
  # @endif
  # bool unregisterListener(ListenerId id);
  def unregisterListener(self, id):
    guard = OpenRTM_aist.ScopedLock(self._taskMutex)
    len_ = len(self._tasks)
    for i in range(len_):
      idx = (len_ - 1) - i
      if self._tasks[idx].listener == id:
        del self._tasks[idx]
        return True
    return False


  ##
  # @if jp
  # @class Task
  # @brief ѥ饹
  # @else
  #
  # @endif
  class Task:
    def __init__(self, lb, tm):
      self.listener = lb
      self.period = tm
      self.remains = tm
